/* gnapster.c - controls main loop and various run-time GUI changes */

#include <stdio.h>

#include <sys/stat.h>
#include <unistd.h>
#include <netinet/in.h>

#include <signal.h>

#include <gnome.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "callbacks.h"

#include "upload.h"
#include "network.h"

#include "config.h"

GNAPSTER_MAIN gnapster_main;
SEARCH_TAB search_tab;
DOWNLOAD_TAB download_tab;
UPLOAD_TAB upload_tab;
CONSOLE_TAB console_tab;
CDDB_TAB cddb_tab;
MOTD_TAB motd_tab;

PROP_DLG prop_dlg;
BUILD_DLG build_dlg;
BROWSE_DLG browse_dlg;
ADD_DLG add_dlg;
EDIT_DLG edit_dlg;

CONN_INFO conn_info;
USER_INFO user_info;

int building;
int submit_index = 0;
GList *total_list = NULL;

CONFIG_TABLE conf_table[] = {
  /* Key                        Default Value */
   { "name_fix", 		1 },
   { "theme_colors", 		0 },
   { "auto_rm", 		0 },
   { "opennap_server", 		0 },
   { "auto_start_queue", 	1 },
   { "line_speed_switch", 	0 },
   { "line_speed_cmp", 		0 },
   { "line_speed", 		0 },
   { "bitrate_switch", 		0 },
   { "bitrate_cmp", 		0 },
   { "bitrate", 		0 },
   { "max_uploads_switch", 	1 },
   { "max_uploads", 		5 },
   { "reject_ul", 		0 },
   { "firewall", 		0 },
   { NULL, 		       -1 },
};

void conv_file(char *s) {
   int i = 0;

   if (!user_info.conf[NAME_FIX]) return;
   
   while(s[i]) {
      if (s[i] == ' ') s[i] = '_';
      i++;
   }
}

FILE *open_file(NAP_DOWNLOAD *download, char *file) {
   FILE *f;
   char *temp_str, *temp_str2 = NULL;
   int inc = 1;
   
   temp_str = g_strdup_printf("%s/%s", user_info.download_dir, file);
   conv_file(temp_str);
   if (file_exists(temp_str)) {
      temp_str2 = g_strdup_printf("%s.%i", temp_str, inc);
      conv_file(temp_str2);
      while(file_exists(temp_str2)) {
	 g_free(temp_str2);
	 temp_str2 = g_strdup_printf("%s.%i", temp_str, inc);
	 conv_file(temp_str2);
	 inc++;
      }
   } else temp_str2 = strdup(temp_str);
   
   f = fopen(temp_str2, "w");
   download->fpath = strdup(temp_str2);
   
   if (strcmp(temp_str, temp_str2)) {
      char *temp_str3;
      temp_str3 = g_strdup_printf("File already exists, using backup file %s...\n",
				  temp_str2);
      gnapster_text_insert(CONSOLE, temp_str3);
      g_free(temp_str3);
   }
   
   g_free(temp_str);
   g_free(temp_str2);
   return f;
}

int file_exists(char *file) {
   struct stat st;
   int x;
   
   x = stat(file, &st);
   if (x < 0) return 0;
   
   return 1;
}

/* Totally destroys an MP3 GList */
GList *mp3_list_free(GList *list) {
   GList *ptr;
   MP3 *mp3;
   
   if (!list) return NULL;

   ptr = list;
   while(ptr) {
      mp3 = (MP3 *)ptr->data;
      if (mp3) {
	 g_free(mp3->filename);
	 g_free(mp3->checksum);
	 g_free(mp3);
	 ptr->data = NULL;
      }
      list = g_list_remove_link(list, ptr);
      ptr = ptr->next;
   }
   
   return list;
}

char *next_arg_full(char *s, char **next, char end) {
   char *ptr, *str;
   
   /* Checks if the entire string is NULL or that we've pushed the
    * string to become the \0 */
   if (!s || !*s) return NULL;
   
   ptr = s;
   
   str = (end == '"') ? ++ptr : ptr;
   while(*str && *str != end && *str != '\n')
     str++;
   if (*str) {
      *str++ = 0;
      if (end == '"') str++;
   }
   
   if (next)
     *next = str;
   
   return ptr;
}

char *next_arg(char *s, char **next) {
   char cmd;
   
   if (*s == '"') cmd = '"';
   else cmd = ' ';
   
   return next_arg_full(s, next, cmd);
}

char *last_arg(char *s, char **next) {
   return next_arg_full(s, next, 0);
}

void queue_check(char *dl, int next) {
   char *dl2;
   
   if (!g_list_length(GTK_CLIST(download_tab.clist)->row_list) &&
       !user_info.queue &&
       user_info.conf[AUTO_START_QUEUE]) {
      start_queue_cb();
   }
   
   if (next == -1) return;
   
   if (dl && conn_info.curr_queue)
     if (strcmp(dl, conn_info.curr_queue)) return;

   if (!dl) return;
   dl2 = strdup(dl);
   rem_queue_timeout(dl2);
   gtk_clist_remove(GTK_CLIST(download_tab.queue_clist), 0);
   if (next) queue_download_handle(dl2);
   g_free(dl2);
}

void rem_queue_timeout(char *dl) {
   NAP_SEARCH *temp;

   if (dl && conn_info.curr_queue)
     if (strcmp(dl, conn_info.curr_queue)) return;

   temp = gtk_clist_get_row_data(GTK_CLIST(download_tab.queue_clist), 0);
   if (!temp) return;
   gtk_timeout_remove(temp->timeout_tag);
}

int queue_timeout(void *data) {
   char *dl;
   NAP_SEARCH *temp;
   
   if (!data) return 0;
   
   temp = (NAP_SEARCH *)data;
   if (!temp->file) return 0;
   dl = strdup(temp->file);
   gtk_clist_remove(GTK_CLIST(download_tab.queue_clist), 0);
   queue_download_handle(dl);
   g_free(dl);
   
   return 0;
}

void queue_download_handle(char *dl) {
   GtkWidget *clist = (GtkWidget *)download_tab.queue_clist;
   NAP_SEARCH *temp;
   
   if (dl && conn_info.curr_queue)
     if (strcmp(dl, conn_info.curr_queue)) 
       return;

   if (!user_info.queue) return;
   if (!g_list_length(GTK_CLIST(clist)->row_list)) {
      stop_queue_cb();
      return;
   }

   temp = gtk_clist_get_row_data(GTK_CLIST(clist), 0);
   if (!temp) return;
      
   gnapster_text_insert(CONSOLE, "Sending download request to %s...\n",
			temp->user);
   napster_send(conn_info.sock, NAPSTER_GET, "%s \"%s\"",
		temp->user, temp->file);

   gtk_clist_set_text(GTK_CLIST(clist), 0, 1, _("Requesting..."));
   gtk_label_set_text(GTK_LABEL(download_tab.queue_label),
		      "");
   
   g_free(conn_info.curr_queue);
   conn_info.curr_queue = strdup(temp->file);
   
   temp->timeout_tag = gtk_timeout_add(180000, queue_timeout, temp);
}

NAP_SEARCH *search_dup(NAP_SEARCH *search) {
   NAP_SEARCH *temp;
   
   if (!search) return NULL;
   
   temp = g_malloc(sizeof(NAP_SEARCH));
   memset(temp, 0, sizeof(NAP_SEARCH));
   temp->type = N_SEARCH;
   temp->trunc_file = strdup(search->trunc_file);
   temp->file = strdup(search->file);
   temp->user = strdup(search->user);
   temp->id = strdup(search->id);
   temp->num = search->num;
   temp->size = search->size;
   temp->kbps = search->kbps;
   temp->freq = search->freq;
   temp->connection = search->connection;
   temp->ip = search->ip;
   
   return temp;
}

long my_atol(const char *nptr) {
   if (!nptr) return -1;
   
   return atol(nptr);
}

int my_atoi(const char *nptr) {
   if (!nptr) return -1;
   
   return atoi(nptr);
}

NAP_DOWNLOAD *gnapster_parse_download(char *s, int alt) {
   char *trunc_file, *file, *user, *id, *ip_str;
   int port, connection;
   unsigned long int ip = 0;
   NAP_DOWNLOAD *temp;
   
   user = next_arg(s, &s);
//   ip = my_atol(next_arg(s, &s));
   ip_str = next_arg(s, &s);
   port = my_atoi(next_arg(s, &s));
   file = next_arg(s, &s);
   id = next_arg(s, &s);
   connection = my_atoi(next_arg(s, &s));
   if (connection < 0) return NULL;
   
   sscanf(ip_str, "%lu", &ip);
   
   trunc_file = strrchr(file, '\\');
   if (!trunc_file) return NULL;
   trunc_file++;

   if (port == 0 && user_info.conf[FIREWALL] && !alt) {
      gnapster_text_insert(CONSOLE, "Unable to receive from %s, user cannot accept connections\n",
			   user);
      return NULL;
   }
   
   temp = g_malloc(sizeof(NAP_DOWNLOAD));
   memset(temp, 0, sizeof(NAP_DOWNLOAD));
   temp->type = N_DOWNLOAD;
   temp->trunc_file = strdup(trunc_file);
   temp->file = strdup(file);
   temp->user = strdup(user);
   temp->fpath = NULL;
   temp->connection = connection;
   temp->port = port;
   temp->ip = nap_naptohl(ip);
   temp->firewall = alt;
   
   return temp;
}

NAP_SEARCH *gnapster_parse_search(char *s) {
   char *trunc_file, *file, *user, *id, *ip_str;
   int num, size, kbps, freq, connection;
   unsigned long int ip;
   NAP_SEARCH *temp;

   file = next_arg(s, &s);
   id = next_arg(s, &s);
   size = my_atoi(next_arg(s, &s));
   kbps = my_atoi(next_arg(s, &s));
   freq = my_atoi(next_arg(s, &s));
   num = my_atoi(next_arg(s, &s));
   user = next_arg(s, &s);
//   ip = my_atol(next_arg(s, &s));
   ip_str = next_arg(s, &s);
   connection = my_atoi(next_arg(s, &s));
   if (connection < 0) return NULL;
   
   sscanf(ip_str, "%lu", &ip);
   
   trunc_file = strrchr(file, '\\');
   if (!trunc_file) return NULL;
   trunc_file++;
   
   temp = g_malloc(sizeof(NAP_SEARCH));
   memset(temp, 0, sizeof(NAP_SEARCH));
   temp->type = N_SEARCH;
   temp->trunc_file = strdup(trunc_file);
   temp->file = strdup(file);
   temp->user = strdup(user);
   temp->id = strdup(id);
   temp->num = num;
   temp->size = size;
   temp->kbps = kbps;
   temp->freq = freq;
   temp->connection = connection;
   temp->ip = ip;
   
   return temp;
}

NAP_SEARCH *gnapster_parse_browse(char *s) {
   char *user, *file, *id;
   int size, kbps, freq, num;
   char *trunc_file;
   NAP_SEARCH *temp;

   user = next_arg(s, &s);
   file = next_arg(s, &s);
   id = next_arg(s, &s);
   size = my_atoi(next_arg(s, &s));
   kbps = my_atoi(next_arg(s, &s));
   freq = my_atoi(next_arg(s, &s));
   num = my_atoi(next_arg(s, &s));
   if (num < 0) return NULL;
   
   trunc_file = strrchr(file, '\\');
   if (!trunc_file) return NULL;
   trunc_file++;
   
   temp = g_malloc(sizeof(NAP_SEARCH));
   memset(temp, 0, sizeof(NAP_SEARCH));
   temp->type = N_SEARCH;
   temp->trunc_file = strdup(trunc_file);
   temp->file = strdup(file);
   temp->user = strdup(user);
   temp->id = strdup(id);
   temp->num = num;
   temp->size = size;
   temp->kbps = kbps;
   temp->freq = freq;
   
   return temp;
}

int gnapster_update_appbar() {
   float val;
   gnapster_main.progress = gnome_appbar_get_progress(GNOME_APPBAR(gnapster_main.appbar));
   val = gtk_progress_get_value(GTK_PROGRESS(gnapster_main.progress));
   val += 0.5;
   if (val > 100) val = 0;
   gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), val);
   
   return 1;
}

void gnapster_update_status(char *s, int i) {
   if (s) {
      gnome_appbar_pop(GNOME_APPBAR(gnapster_main.appbar));
      gnome_appbar_push(GNOME_APPBAR(gnapster_main.appbar), s);
   }
   if (i == 0) {
      gtk_timeout_remove(gnapster_main.appbar_timeout);
   } else if (i == 1) {
      if (gnapster_main.appbar_timeout) gtk_timeout_remove(gnapster_main.appbar_timeout);
      gnapster_main.appbar_timeout = gtk_timeout_add(50, gnapster_update_appbar,
						     NULL);
   }
}

void gnapster_highlight(GtkWidget *notebook, GtkWidget *vbox) {
   GtkWidget *label, *notebook_page;
   int curr_page;
   
   label = gtk_notebook_get_tab_label(GTK_NOTEBOOK(notebook),
				      vbox);
   curr_page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
   notebook_page = gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), 
					     curr_page);
   if (notebook_page != GTK_WIDGET(vbox))
     gtk_widget_set_style(label, gnapster_main.highlight);   
}

char *simplify_size(unsigned long int size) {
   float new_size = size;
   int div_count = 0;
   static char buf[25];
   char lbl;
   
   while(new_size >= 1024) {
      new_size /= 1024;
      div_count++;
   }
   
   switch(div_count) {
    case 0:
      lbl = 'b';
      break;
    case 1:
      lbl = 'k';
      break;
    case 2:
      lbl = 'M';
      break;
    case 3:
      lbl = 'G';
      break;
    default:
      lbl = '?';
      break;
   }
   
   g_snprintf(buf, sizeof(buf), "%.02f%c", new_size, lbl);
   
   return buf;
}

int chan_lookup(char *chan) {
   GList *ptr;
   int i = 0;
   
   ptr = console_tab.channel_list;
   while(ptr) {
      if (!g_strcasecmp(((CHANNEL *)ptr->data)->channel_name, chan))
	return (i + 2);
      i++;
      ptr = ptr->next;
   }
   
   return -1;
}

GtkCList *chan_lookup_clist(char *chan) {
   GList *ptr;
   
   ptr = console_tab.channel_list;
   while(ptr) {
      if (!g_strcasecmp(((CHANNEL *)ptr->data)->channel_name, chan))
	return GTK_CLIST(((CHANNEL *)ptr->data)->clist);
      ptr = ptr->next;
   }
   
   return NULL;
}

char *chan_lookup_chan(int index) {
   CHANNEL *chan;
   
   chan = g_list_nth_data(console_tab.channel_list, index);
   
   if (chan)
     return chan->channel_name;
   else return NULL;
}

char *text_insert_next_val(char **s, int *col) {
   char *ptr, *str;
   
   if (!*s || !**s) return NULL;
   
   ptr = *s;
   str = ptr;
   while(*str) {
      if (*str >= 1 && *str <= 3) {
	 *col = *str;
	 break;
      }
      str++;
   }
   
   if (*str)
     *str++ = 0;
   
   if (s)
     *s = str;
   
   return ptr;
}

void gnapster_real_text_insert(GtkWidget *text, char *fmt) {
   char *ptr;
   int col, next_col = 1;
   GdkColor colors[] = {
      { 0, 0x0000, 0x0000, 0x0000 },
      { 0, 0x0000, 0x0000, 0x7FFF },
      { 0, 0x7FFF, 0x0000, 0x0000 },
      { 0, 0x0000, 0x0000, 0x0000 },
      { 0, 0xaaaa, 0x0000, 0x0000 },
   };
   GdkColor *true_col;
   
   if (!fmt) return;
   
   ptr = fmt;
   while(ptr) {
      col = next_col;
      ptr = text_insert_next_val(&fmt, &next_col);
      if (!ptr) break;
      true_col = &colors[col - 1];
      if (user_info.conf[THEME_COLORS]) {
	 if (col == 1) 
	   true_col = &gnapster_main.normal->fg[GTK_STATE_NORMAL];
	 else if (col == 2)
	   true_col = &gnapster_main.highlight->fg[GTK_STATE_NORMAL];
      }
      gtk_text_insert(GTK_TEXT(text), NULL, true_col, NULL, ptr, -1);
   }
}

void gnapster_text_insert(int val, char *fmt, ...) {
   GtkWidget *text, *vbox, *parent;
   char *format_str;
   va_list args;
   
   if (val < 0) return;
   
   va_start(args, fmt);
   format_str = g_strdup_vprintf(fmt, args);
   va_end(args);

   parent = console_tab.notebook;
   switch(val) {
    case CONSOLE:
      text = console_tab.text;
      vbox = console_tab.vbox;
      break;
    case MOTD:
      parent = gnapster_main.notebook;
      text = motd_tab.text;
      vbox = motd_tab.vbox;
      break;
    default:
	{
	   CHANNEL *chan;
	   
	   chan = g_list_nth_data(console_tab.channel_list, (val - 2));
	   if (!chan) printf("uhm\n");

	   text = chan->text;
	   vbox = chan->vbox;
	}
      break;
   }

   gnapster_real_text_insert(text, format_str);
   
   g_free(format_str);
   if (parent == console_tab.notebook)
     gnapster_highlight(gnapster_main.notebook, console_tab.main_vbox);
   gnapster_highlight(parent, vbox);
}

void download_free(NAP_DOWNLOAD *download) {
   g_free(download->trunc_file);
   g_free(download->file);
   g_free(download->fpath);
   g_free(download->user);
   g_free(download);
}

void search_free(NAP_SEARCH *search) {
   g_free(search->trunc_file);
   g_free(search->file);
   g_free(search->user);
   g_free(search->id);
   g_free(search);
}

void server_free(NAP_SERVER *data) {
   g_free(data->ip);
   g_free(data->desc);
   g_free(data);
}

void gnapster_destroy(gpointer data) {
   TYPE *n = (TYPE *)data;
   
   if (!data) return;
   
   if (n->type == N_SEARCH)
     search_free((NAP_SEARCH *)data);
   else if (n->type == N_DOWNLOAD)
     download_free((NAP_DOWNLOAD *)data);
   else if (n->type == N_SERVER)
     server_free((NAP_SERVER *)data);
   else
     g_free(data);
}

void gnapster_clist_append(GtkWidget *clist, GtkWidget *vbox, void *data, char *s1, char *s2,
			   char *s3, char *s4, char *s5) {
   char *add_id[6] = { s1, s2, s3, s4, s5, NULL };
   int i, row;
   
   if (!data) return;
   
   for(i=0; i<5; i++) {
      if (!add_id[i]) add_id[i] = "???";
   }
   
   row = gtk_clist_append(GTK_CLIST(clist), add_id);
   
   gtk_clist_set_row_data_full(GTK_CLIST(clist), row, data,
			       (GtkDestroyNotify)gnapster_destroy);

   if (vbox)
     gnapster_highlight(gnapster_main.notebook, vbox);
}

void *gnapster_get_selection_data(GtkWidget *clist, int *row) {
   void *temp;
   GList *selection;
   
   selection = GTK_CLIST(clist)->selection;
   if (!selection) return NULL;

   temp = gtk_clist_get_row_data(GTK_CLIST(clist), (int)selection->data);
   if (row)
     *row = (int)selection->data;
   
   return temp;
}

void gnapster_get_styles(GtkWidget *widget) {
   gnapster_main.normal = gtk_widget_get_style(widget);
   gnapster_main.highlight = gtk_style_new();
   gnapster_main.highlight->font = gnapster_main.normal->font;
   gnapster_main.highlight->fg[GTK_STATE_NORMAL] =
     gnapster_main.normal->fg[GTK_STATE_SELECTED];
}

void gnapster_config_set_int(int defaults) {
   char temp_str[1024];
   int i = 0;

   while(conf_table[i].conf_str) {
      g_snprintf(temp_str, sizeof(temp_str), "/gnapster/Options/%s",
		 conf_table[i].conf_str);
      gnome_config_set_int(temp_str, 
			   (defaults == 1) ? conf_table[i].conf_val : user_info.conf[i]);
      i++;
   }
   gnome_config_sync();
}

void gnapster_config_get_int() {
   char temp_str[1024];
   int i = 0;
   
   if (!gnome_config_has_section("/gnapster/Options")) return;
   
   while(conf_table[i].conf_str) {
      g_snprintf(temp_str, sizeof(temp_str), "/gnapster/Options/%s",
		 conf_table[i].conf_str);
      user_info.conf[i] = gnome_config_get_int(temp_str);
      i++;
   }
}

int verify_shared() {
   FILE *f;
   char *ptr, *str, buf[256];
   int args = 0;
   
   f = open_shared(0);
   /* Return 1 to just ignore it */
   if (!f) return 1;
   /* Read the first line only */
   fgets(buf, sizeof(buf) - 1, f);
   fclose(f);
   str = buf;
   do {
      ptr = next_arg(str, &str);
      args++;
   } while(ptr);
   if (args <= 7) return 0;
   
   return 1;
}

int idle_startup_cb(gpointer data) {
//   printf("data -> %s\n", (char *)data);
   if (!strcmp(data, "connect"))
     connect_cb();
   else if (!strcmp(data, "props"))
     preferences_cb();
   return 0;
}

int main(int argc, char **argv) {
//   bindtextdomain(PACKAGE, GNOMELOCALEDIR);
//   textdomain(PACKAGE);
   
   gnome_init(PACKAGE, VERSION, argc, argv);

   gnapster_config_get_int();

   create_gnapster_main();
   
   create_search_tab();
   create_download_tab();
   create_upload_tab();
   create_console_tab();
/*   create_cddb_tab();*/
   create_motd_tab();
   
   gtk_widget_show_all(gnapster_main.window);
   gnapster_get_styles(gnapster_main.window);

   conn_info.connected = conn_info.connecting =
     conn_info.searching = building = 0;

   memset(&user_info, 0, sizeof(USER_INFO));
   user_info.conf[TERMINATION] = -1;
   conn_info.curr_queue = NULL;
   
   browse_dlg.window = NULL;
   
   user_info.user = gnome_config_get_string("/gnapster/User/user");
   if (!user_info.user) {
      gnapster_update_status(_("Please fill out the preferences form before trying to connect."), -1);
      gtk_idle_add(idle_startup_cb, "props");
   } else {
      gnapster_update_status(_("Auto-connecting..."), -1);
      user_info.pass = gnome_config_get_string("/gnapster/User/pass");
      user_info.download_dir = gnome_config_get_string("/gnapster/User/dl_dir");
      user_info.upload_dir = gnome_config_get_string("/gnapster/User/ul_dir");
      user_info.connection = gnome_config_get_int("/gnapster/User/connection");
      user_info.server = gnome_config_get_string("/gnapster/Options/server");
      if (!gnome_config_has_section("/gnapster/Options")) {
	 user_info.conf[THEME_COLORS] = 0;
	 gnapster_text_insert(CONSOLE,
			      "Options category not set, assuming defaults.  Please check the preferences dialog for the new preference changes\n");
	 gnapster_config_set_int(1);
      }
      gnapster_config_get_int();
      user_info.media = strdup("Audio");
      gtk_idle_add(idle_startup_cb, "connect");
   }

   if (!user_info.conf[FIREWALL]) {
      conn_info.nap_port = gnome_config_get_int("/gnapster/Options/port");
      if (conn_info.nap_port)
	napster_bind(conn_info.nap_port);
      else {
	 gnome_config_set_int("/gnapster/Options/port", NAP_PORT);
	 gnome_config_sync();
	 conn_info.nap_port = NAP_PORT;
	 napster_bind(NAP_PORT);
      }
   } else {
      /* Don't bind */
      conn_info.nap_port = 0;
   }
   
   signal(SIGPIPE, SIG_IGN);

   if (!verify_shared())
     gnome_warning_dialog("Your ~/.gnapster_shared file is outdated, please use the preferences menu to rebuild your list");
   
   gtk_main();
   
   return 1;
}
