#ifndef __GNAPSTER_H
#define __GNAPSTER_H

#define NAP_VERSION "Gnapster 1.3.7"
/* Please note that this port will be held in ~/.gnome/gnapster upon
 * execution of Gnapster and this value will only be used when the config
 * file cannot be found */
#define NAP_PORT 6699

/* I don't think you really wanna do this unless you like pointless messages */
//#define GNAPSTER_DEBUG

#ifdef GNAPSTER_DEBUG
#define DPRINTF(x) do { printf x; } while(0)
#else
#define DPRINTF(x) do { ; } while(0)
#endif /* GNAPSTER_DEBUG */

#define CONSOLE 0
#define MOTD 1

typedef struct _config_table {
   char *conf_str;
   int conf_val;
} CONFIG_TABLE;

typedef struct {
   int type;
} TYPE;

typedef struct _channel {
   char *channel_name;
   GtkWidget *text;
   GtkWidget *clist;
   GtkWidget *vbox;
} CHANNEL;

typedef struct _conn_info {
   int sock;
   int sock_input;
   int bind_sock;
   int bind_sock_input;
   int cddb_sock;
   int cddb_sock_input;
   int upload_input;
   struct sockaddr_in saddr;
   int timeout;
   int connecting;
   int connected;
   char *status;
   char *ip;
   int port;
   int nap_port;
   int searching;
   char *curr_queue;
} CONN_INFO;

#define NAME_FIX 0
#define THEME_COLORS 1
#define AUTO_RM 2
#define OPENNAP_SERVER 3
#define AUTO_START_QUEUE 4
#define LINE_SPEED_SWITCH 5
#define LINE_SPEED_CMP 6
#define LINE_SPEED 7
#define BITRATE_SWITCH 8
#define BITRATE_CMP 9
#define BITRATE 10
#define MAX_UPLOADS_SWITCH 11
#define MAX_UPLOADS 12
#define REJECT_UL 13
#define FIREWALL 14
#define TERMINATION 15

typedef struct _user_info {
   char *user;
   char *pass;
   char *email;
   char *download_dir;
   char *upload_dir;
//   char *server;
   int connection;
   int conf[TERMINATION + 1];
   int new_user;
   int sharing;
   double share_size;
   char *media;
   int queue;
   char *server;
   int server_port;
} USER_INFO;

void conv_file(char *);
FILE *open_file(NAP_DOWNLOAD *, char *);
GList *mp3_list_free(GList *);
int file_exists(char *);
long my_atol(const char *);
int my_atoi(const char *);
char *next_arg_full(char *, char **, char);
char *next_arg(char *, char **);
char *last_arg(char *, char **);
void queue_check(char *, int);
void rem_queue_timeout(char *);
int queue_timeout(void *);
void queue_download_handle(char *);
NAP_SEARCH *search_dup(NAP_SEARCH *);
NAP_DOWNLOAD *gnapster_parse_download(char *, int);
NAP_SEARCH *gnapster_parse_search(char *);
NAP_SEARCH *gnapster_parse_browse(char *);
int gnapster_update_appbar();
void gnapster_update_status(char *, int);
void gnapster_highlight(GtkWidget *, GtkWidget *);
char *simplify_size(unsigned long int);
int chan_lookup(char *);
GtkCList *chan_lookup_clist(char *);
char *chan_lookup_chan(int);
char *text_insert_next_val(char **, int *);
void gnapster_real_text_insert(GtkWidget *, char *);
void gnapster_text_insert(int, char *, ...);
void download_free(NAP_DOWNLOAD *);
void search_free(NAP_SEARCH *);
void server_free(NAP_SERVER *);
void gnapster_clist_append(GtkWidget *, GtkWidget *, void *, char *, char *, char *, char *, char *);
void *gnapster_get_selection_data(GtkWidget *, int *);
void gnapster_get_styles(GtkWidget *);
void gnapster_config_set_int(int);
void gnapster_config_get_int();

#endif /* __GNAPSTER_H */
