#ifndef __NAPSTER_H
#define __NAPSTER_H

#define NAPSTER_ERROR 0

#define NAPSTER_LOGIN 2
#define NAPSTER_NEW_USER 7
#define NAPSTER_NEWUSER_SUCCESS 8
#define NAPSTER_NEWUSER_REG 9
#define NAPSTER_NEWUSER_INVALID_NICK 10

#define NAPSTER_SUBMIT 100

#define NAPSTER_SEARCH 200
#define NAPSTER_SEARCH_RESULT 201
#define NAPSTER_END_SEARCH 202

#define NAPSTER_GET 203
#define NAPSTER_DOWNLOAD_REPLY 204

#define NAPSTER_USER_MESSAGE 205

#define NAPSTER_DOWNLOAD_ERROR 206

#define NAPSTER_BROWSE_ERROR 210
#define NAPSTER_BROWSE 211
#define NAPSTER_BROWSE_RESULT 212
#define NAPSTER_END_BROWSE 213

#define NAPSTER_STATUS_MESSAGE 214

#define NAPSTER_DOWNLOAD_START 218
#define NAPSTER_DOWNLOAD_COMPLETE 219
#define NAPSTER_UPLOAD_START 220
#define NAPSTER_UPLOAD_COMPLETE 221

#define NAPSTER_JOIN_CHAN 400
#define NAPSTER_PART_CHAN 401
#define NAPSTER_CHAN_MESSAGE 402
#define NAPSTER_CHAN_MESSAGE_IN 403

#define NAPSTER_USER_ERROR 404

#define NAPSTER_CHAN_JOIN 405
#define NAPSTER_CHAN_JOINUSER 406
#define NAPSTER_CHAN_PARTUSER 407
#define NAPSTER_CHAN_USERLIST 408
#define NAPSTER_CHAN_USERLIST_END 409
#define NAPSTER_CHAN_TOPIC 410

#define NAPSTER_ALT_GET 500
#define NAPSTER_ALT_DOWNLOAD_REPLY 501 

#define NAPSTER_WHOIS_REQUEST 603
#define NAPSTER_WHOIS_REPLY 604
#define NAPSTER_WHOIS_OFFLINE 605

#define NAPSTER_UPLOAD_REQUEST 607
#define NAPSTER_ACCEPT_UPLOAD_REQUEST 608

#define NAPSTER_REQUEST_PORTCHANGE 613

#define NAPSTER_SENDLIMIT 619
#define NAPSTER_SENDLIMIT_REACHED 620

#define NAPSTER_MESSAGE 621

#define NAPSTER_CHANGE_PORT 703

#define NAPSTER_PING 751
#define NAPSTER_PONG 752

#define NAPSTER_ACTION 824


#define N_DOWNLOAD 0
#define N_SEARCH 1
#define N_SERVER 2
#define N_FILE 3

typedef struct _nap_download {
   int type;
   char *trunc_file;
   char *file;
   char *user;
   int connection;
   int port;
   unsigned long int ip;
   
   struct sockaddr_in saddr;
   
   int sock;
   int sock_input;
   int update_tag;
   FILE *fdesc;
   char *fpath;
   char header[40];
   int header_val;
   int size, temp_size;
   unsigned long int total;
   int calls;
   float kps;
   int timeout_calls;
   int firewall;
} NAP_DOWNLOAD;

typedef struct _nap_search {
   int type;
   char *file;
   char *trunc_file;
   char *id;
   int size;
   int kbps;
   int freq;
   int num;
   char *user;
   unsigned long int ip;
   int connection;
   int timeout_tag;
} NAP_SEARCH;

typedef struct _nap_server {
   int type;
   char *ip;
   int port;
   char *desc;
} NAP_SERVER;

typedef struct _nap_file {
   int type;
   time_t mtime;
   char *filename;
} NAP_FILE;

typedef struct _cddb_get {
   char *href;
   char *desc;
} CDDB_GET;

void napster_dns_cb(guint32, int *);
void napster_get_best_host();
void napster_connect_cddb();
void napster_connect(char *, int);
void napster_connect_user(NAP_DOWNLOAD *);
void napster_bind(int);
void napster_send_get(int, char *, char *, int);
void napster_send_send(int, char *, char *, unsigned long int);
int napster_send(int, int, char *, ...);
int is_opennap_server(char *);
void napster_handle_data(int, char *);
int napster_conn_as_int(char *);
char *napster_conn_as_str(int);

#endif /* __NAPSTER_H */
