/* network.c - general socket and network code callbacks  */

#include <stdio.h>
#include <stdlib.h>

#include <string.h>
#include <errno.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>

#include <sys/stat.h>
#include <fcntl.h>

#include <unistd.h>
#include <ctype.h>

#include <gnome.h>

#include "napster.h"
#include "network.h"
#include "gnapster.h"
#include "ui.h"
#include "upload.h"
#include "callbacks.h"

#include "config.h"

#define CLIST_UPDATE_SPEED 1000

extern CONN_INFO conn_info;
extern USER_INFO user_info;
extern DOWNLOAD_TAB download_tab;
extern UPLOAD_TAB upload_tab;
extern CONSOLE_TAB console_tab;
extern SEARCH_TAB search_tab;
extern CDDB_TAB cddb_tab;
extern MOTD_TAB motd_tab;

char cddb_read[1024];
int cddb_val = 0;

int packet_len, packet_cmd, total_packet_len, read_count;
char packet_data[1024];

extern GList *total_list;
extern int submit_index;

struct in_addr *network_atoaddr(char *address) {
   struct hostent *host;
   static struct in_addr saddr;
   
   saddr.s_addr = inet_addr(address);
   if (saddr.s_addr != -1) return &saddr;
   host = gethostbyname(address);
   if (host != NULL) return (struct in_addr *)*host->h_addr_list;
   return NULL;
}

/* Don't worry about the CDDB stuff yet, it's an unsupported feature,
 * I'll finish it later :) */
int network_cddb_parse(char *s, char *href, char *text) {
   char *txt, *bs;
   char *hrf, *stuff;
   
   txt = strstr(s, "<!-- START ITEM -->");
   if (!txt) return 0;
   txt += strlen("<!-- START ITEM -->");
   bs = strstr(s, "<!-- END ITEM -->");
   if (!bs) return 0;
   bs[0] = '\0';
   bs++;
   txt = strstr(txt, " -->");
   if (!txt) return 0;
   txt += 4;
   hrf = strstr(txt, "<A HREF=\"");
   if (!hrf) return 0;
   hrf += strlen("<A HREF=\"");
   stuff = strstr(hrf, "\">");
   if (!stuff) return 0;
   stuff[0] = '\0';
   stuff += 2;
   bs = strstr(stuff, "</A>");
   if (!bs) return 0;
   bs[0] = '\0';
   bs += 4;   
   
   sprintf(href, hrf);
   sprintf(text, stuff);
   
   return 1;
}

void network_conn_cddb_read(gpointer data, int source, GdkInputCondition condition) {
   char buf[1024];
   int n, i;
   
   n = recv(source, buf, sizeof(buf) - 1, 0);
   if (n <= 0) {
      printf("Connection closed...\n");
      gtk_clist_thaw(GTK_CLIST(cddb_tab.clist));
      gdk_input_remove(conn_info.cddb_sock_input);
      close(source);
      return;
   }

   for(i=0; i<n; i++) {
      if (buf[i] == '\n') {
	 char href[1024], text[1024];
	 cddb_read[cddb_val] = '\0';
	 cddb_val = 0;
	 if (network_cddb_parse(cddb_read, href, text)) {
	    CDDB_GET *cddb_get;
	    cddb_get = malloc(sizeof(CDDB_GET));
	    cddb_get->href = strdup(href);
	    cddb_get->desc = strdup(text);
	    gnapster_clist_append(cddb_tab.clist, cddb_tab.vbox, cddb_get,
				  cddb_get->desc, NULL, NULL, NULL, NULL);
	 }
      } else cddb_read[cddb_val++] = buf[i];
   }
}

char *network_fix_get(char *s) {
   int i = 0, flag = 0;
   static char buf[1024];
   char chr[4];

   buf[0] = '\0';
   while(s[i]) {
      if (s[i] == '?') flag = 1;
      if (s[i] == ' ' && flag) {
	 sprintf(chr, "+");
      } else if (s[i] == '/' && flag) {
	 sprintf(chr, "%%2F");
      } else sprintf(chr, "%c", s[i]);
      strcat(buf, chr);
      i++;
   }
   
   return buf;
}

void network_conn_cddb_cb(gpointer data, int source, GdkInputCondition condition) {
   char *gs, *get_str;
   gdk_input_remove(conn_info.cddb_sock_input);
   
   if (connect(source, (struct sockaddr *)&conn_info.saddr, sizeof(conn_info.saddr)) < 0) {
      printf("handle later...didn't connect\n");
      close(source);
      return;
   }
   
   get_str = g_strdup_printf("GET /xm/search?q=%s&f=artist\n\n", 
			     gtk_entry_get_text(GTK_ENTRY(cddb_tab.entry)));
   gs = network_fix_get(get_str);
   printf("sending: %s", gs);
   send(source, gs, strlen(gs), 0);
   g_free(get_str);
   
   conn_info.cddb_sock_input = gdk_input_add(source, GDK_INPUT_READ,
					     network_conn_cddb_read, NULL);
}

void network_add_download(NAP_DOWNLOAD *download) {
   gnapster_text_insert(CONSOLE, "Receiving %s from %s...\n",
			download->trunc_file, download->user);
   gnapster_clist_append(download_tab.clist, download_tab.vbox,
			 download, download->trunc_file,
			 napster_conn_as_str(download->connection),
			 download->user, "0.00M [0%] @ 0.0k/s", NULL);
   gtk_label_set_text(GTK_LABEL(download_tab.queue_label), "");
   queue_check(download->file, 0);
   
   napster_send(conn_info.sock, NAPSTER_DOWNLOAD_START, NULL);
   
   download->calls = 5;
   
   download->update_tag = gtk_timeout_add(CLIST_UPDATE_SPEED, network_update_download,
					  download);
   download->sock_input = gdk_input_add(download->sock, GDK_INPUT_READ,
					network_handle_download, download);
}

void network_add_upload(NAP_DOWNLOAD *upload) {
   gnapster_text_insert(CONSOLE, "Sending %s to %s...\n",
			upload->trunc_file, upload->user);
   gnapster_clist_append(upload_tab.clist, upload_tab.vbox,
			 upload, upload->trunc_file,
			 napster_conn_as_str(upload->connection),
			 upload->user, "0.00M [0%] @ 0.0k/s", NULL);
   
   napster_send(conn_info.sock, NAPSTER_UPLOAD_START, NULL);
   
   upload->calls = 5;
   
   upload->update_tag = gtk_timeout_add(CLIST_UPDATE_SPEED, network_update_upload,
					upload);
   
   upload->sock_input = gdk_input_add(upload->sock, GDK_INPUT_WRITE,
				      network_handle_upload, upload);
}

void network_handle_submit(gpointer data, int source, GdkInputCondition condition) {
   GList *list, *link;
   MP3 *mp3;
   int x;

   if (!total_list) {
      submit_index = 0;
      total_list = mp3_list_free(total_list);
      gdk_input_remove(conn_info.upload_input);
      return;
   }
   
   /* 0 because it's going to further break the link each pass */
   list = g_list_nth(total_list, 0);
   if (!list || submit_index == g_list_length(total_list) || !conn_info.connected) {
      submit_index = 0;
      total_list = mp3_list_free(total_list);
      gdk_input_remove(conn_info.upload_input);
      return;
   }
   mp3 = (MP3 *)list->data;
   
   x = napster_send(conn_info.sock, NAPSTER_SUBMIT,
		    "\"%s\" %s %lu %li %lu %lu",
		    mp3->filename, mp3->checksum, mp3->filesize,
		    mp3->bitrate, mp3->freq, mp3->time);
   
   link = g_list_find(total_list, list->data);
   total_list = g_list_remove_link(total_list, link);
   g_free(mp3->filename);
   g_free(mp3->checksum);
   g_free(mp3);
   list->data = NULL;
      
   submit_index++;

   if (x <= 0) {
      printf("x = %i\n", x);
      submit_index = 0;
      total_list = mp3_list_free(total_list);
      gdk_input_remove(conn_info.upload_input);
      return;
   }
}

void network_handle_sigpipe(int sig) {
   printf("SIGPIPE received\n");
}

void network_handle_accept_header(gpointer data, int source, GdkInputCondition condition) {
   char c;
   int n;
   NAP_DOWNLOAD *download;
   
   download = (NAP_DOWNLOAD *)data;
   
   n = recv(source, &c, 1, 0);
   if (n <= 0 || download->header_val > 4) {
      printf("download->header_val = %i\n",
	     download->header_val);
      close(download->sock);
      download_free(download);
      gdk_input_remove(download->sock_input);
   } else if (n > 0) {
      download->header[download->header_val++] = c;
      download->header[download->header_val] = '\0';
      if (!strcmp(download->header, "GET")) {
	 char buf[4096];
	 struct stat st;
	 char *temp_str;
	 char *user, *file, *trunc_file;
	 int conn, uploads;

	 gdk_input_remove(download->sock_input);

	 DPRINTF(("ACCEPT: %s\n", download->header));
	 
	 bzero(buf, sizeof(buf));
	 n = recv(download->sock, buf, 4096, 0);
	 if (n <= 0) {
	    gnapster_text_insert(CONSOLE, "* Error: Bad header received from %s...\n");
	    close(download->sock);
	    download_free(download);
	 }
	 buf[n] = '\0';
	 temp_str = buf;
	 user = next_arg(temp_str, &temp_str);
	 file = next_arg(temp_str, &temp_str);
	 conn = my_atoi(next_arg(temp_str, &temp_str));
	 if (conn < 0) {
	    gnapster_text_insert(CONSOLE, "* Error: Bad header received from %s...\n",
				 user);
	    close(download->sock);
	    download_free(download);
	    return;
	 }

	 download->user = strdup(user);
	 download->file = strdup(file);
	 download->fpath = NULL;
	 download->connection = conn;

	 trunc_file = strrchr(file, '\\');
	 if (!trunc_file) {
	    printf("Bad header\n");
	    close(download->sock);
	    download_free(download);
	    return;
	 }
	 trunc_file++;
	 download->trunc_file = strdup(trunc_file);

	 uploads = g_list_length(GTK_CLIST(upload_tab.clist)->row_list);
	 	 
	 if (user_info.conf[MAX_UPLOADS_SWITCH] &&
	     uploads >= user_info.conf[MAX_UPLOADS]) {
	    gnapster_text_insert(CONSOLE, "Rejecting upload request for %s to %s, max uploads reached\n",
				 download->trunc_file, download->user);
	    close(download->sock);
	    download_free(download);
	    return;
	 } else if (user_info.conf[REJECT_UL] &&
		    g_list_length(GTK_CLIST(download_tab.clist)->row_list)) {
	    gnapster_text_insert(CONSOLE, "Rejecting upload request for %s to %s, active downloads found\n",
				 download->trunc_file, download->user);
	    close(download->sock);
	    download_free(download);
	    return;
	 }

	 download->fdesc = fopen(conv_unix(download->file), "r");
	 if (!download->fdesc) {
	    temp_str = g_strdup_printf("FILE NOT FOUND");
	    send(download->sock, temp_str, strlen(temp_str), 0);
	    g_free(temp_str);
	    close(download->sock);
	    download_free(download);
	    return;
	 }

	 fstat(fileno(download->fdesc), &st);
	 download->size = 0;
	 download->total = st.st_size;

	 temp_str = g_strdup_printf("%lu", download->total);
	 send(download->sock, temp_str, strlen(temp_str), 0);
	 g_free(temp_str);
	 
	 network_add_upload(download);
      } else if (!strcmp(download->header, "SEND")) {
	 char buf[4096 + 1];
	 struct stat st;
	 char *temp_str;
	 char *user, *file, *trunc_file;
	 int size;

	 gdk_input_remove(download->sock_input);

	 DPRINTF(("ACCEPT: %s\n", download->header));
	 bzero(buf, sizeof(buf));
	 n = recv(download->sock, buf, 4096, 0);
	 buf[n] = '\0';
	 temp_str = buf;
	 user = next_arg(temp_str, &temp_str);
	 file = next_arg(temp_str, &temp_str);
	 size = my_atoi(next_arg(temp_str, &temp_str));
	 if (size < 0) {
	    gnapster_text_insert(CONSOLE, "* Error: Bad header received from %s...\n",
				 user);
	    close(download->sock);
	    download_free(download);
	    return;
	 }
	 
	 download->user = strdup(user);
	 download->file = strdup(file);
	 download->total = size;
	 download->fpath = NULL;

	 trunc_file = strrchr(file, '\\');
	 if (!trunc_file) {
	    printf("Bad header\n");
	    close(download->sock);
	    download_free(download);
	    return;
	 }
	 trunc_file++;
	 download->trunc_file = strdup(trunc_file);
      
	 download->fdesc = open_file(download, download->trunc_file);
	 if (!download->fdesc) {
	    gnome_error_dialog(_("Uh oh.  The path you've chosen does not exist or you do not have write permissions\nPlease open up gnapster's preferences and pick a directory you know you have access to"));
	    close(download->sock);
	    download_free(download);
	    return;
	 }

	 fstat(fileno(download->fdesc), &st);
	 temp_str = g_strdup_printf("%i", (int)st.st_size);
	 send(download->sock, temp_str, strlen(temp_str), 0);
	 g_free(temp_str);
	 download->size = 0;
	 download->connection = 0;
      
	 network_add_download(download);
      }
   }
}

void network_handle_connection(gpointer data, int source, GdkInputCondition condition) {
   NAP_DOWNLOAD *download;
   struct sockaddr_in from;
   int len;
   
   download = g_malloc0(sizeof(NAP_DOWNLOAD));
   download->type = N_DOWNLOAD;
   len = sizeof(from);
   download->sock = 
     accept(conn_info.bind_sock, (struct sockaddr *)&from, &len);
   if (download->sock < 0) {
      printf("accept error\n");
      return;
   }
   send(download->sock, "1", 1, 0);
   download->header_val = 0;
   
   download->sock_input = gdk_input_add(download->sock, GDK_INPUT_READ,
					network_handle_accept_header, download);
}

void download_close(NAP_DOWNLOAD *download, TRANSFER_INFO info) {
   char *temp_str = NULL;
   int incomplete = 0, row;

   gdk_input_remove(download->sock_input);
   gtk_timeout_remove(download->update_tag);
   fclose(download->fdesc);
   
   if (info & TRANSFER_FILE_REMOVE) unlink(download->fpath);
   
   close(download->sock);
   napster_send(conn_info.sock, NAPSTER_DOWNLOAD_COMPLETE, NULL);

   if (info & TRANSFER_FINISHED)
     temp_str = g_strdup_printf("Download complete for %s\n",
				download->trunc_file);
   else if (info & TRANSFER_CLOSED) {
      temp_str = g_strdup_printf("Download cancelled by %s for %s (%s lost)...\n",
				 download->user, download->trunc_file, simplify_size(download->total - download->size));
      incomplete = 1;
   } else if (info & TRANSFER_TIMEOUT) {
      temp_str = g_strdup_printf("Download timed out for %s from %s\n",
				 download->trunc_file, download->user);
      incomplete = 1;
   } else if (info & TRANSFER_CANCEL) {
      incomplete = 1;
   }
   
   if (incomplete) {
      char *path, *file;
      char *new_path;
      
      path = strdup(download->fpath);
      file = strrchr(path, '/');
      if (file) {
	 int x = -1;
	 struct stat st;
	 
	 file[0] = '\0';
	 file++;
	 new_path = g_strdup_printf("%s/incomplete", path);
	 if (stat(new_path, &st) < 0)
	   x = mkdir(new_path, 0755);
	 if (S_ISDIR(st.st_mode) || x >= 0) {
	    g_free(new_path);
	    new_path = g_strdup_printf("%s/incomplete/%s", path, file);
	    rename(download->fpath, new_path);
	 }
	 g_free(new_path);
      }
      g_free(path);
   }
   
   if (temp_str) {
      gnapster_text_insert(CONSOLE, temp_str);
      g_free(temp_str);
   }

   queue_download_handle(download->file);

   row = gtk_clist_find_row_from_data(GTK_CLIST(download_tab.clist), download);
   if (row >= 0)
     gtk_clist_remove(GTK_CLIST(download_tab.clist), row);
  
   queue_check(NULL, -1);
}

void network_handle_download(gpointer data, int source, GdkInputCondition condition) {
   char buf[2048];
   int n;
   NAP_DOWNLOAD *download;
   
   download = (NAP_DOWNLOAD *)data;
   
   bzero(buf, sizeof(buf));
   n = recv(source, buf, 2048, 0);
   if (n <= 0) {
      /* Could someone explain to me why this occurs? */
      if (download->size + 4 == download->total)
	download->size += 4;
      
      if (download->size == download->total)
	download_close(download, TRANSFER_FINISHED);
      else
	download_close(download, TRANSFER_CLOSED);

      return;
   } else if (n > 0) {
      if (fwrite(buf, n, sizeof(char), download->fdesc) <= 0) {
	 gnapster_text_insert(CONSOLE, "Error writing to disk!\n");
	 download_close(download, TRANSFER_CANCEL);
	 return;
      }
      download->size += n;
      download->temp_size += n;
   }
}

int network_update_download(gpointer data) {
   char *temp_str;
   NAP_DOWNLOAD *download;
   
   if (!data) return 0;   
   download = (NAP_DOWNLOAD *)data;
   
   download->calls++;
   if (download->calls >= 3) { /* x seconds */
      if (download->temp_size == 0)
	download->timeout_calls++;
      else
	download->timeout_calls = 0;

      download->kps = 
	(((float)download->temp_size / (float)1024)) / (float)(CLIST_UPDATE_SPEED / 1000);
      download->temp_size = 0;
      download->calls = 0;
      
      if (download->timeout_calls == ((5 * 60) / 3)) {
	 download_close(download, TRANSFER_TIMEOUT);
	 return 0;
      }
   }
   
   temp_str = g_strdup_printf("%.02fM [%i%%] @ %.01fk/s",
			      ((float)download->size / 1024) / 1024,
			      (int)((float)download->size / (float)download->total * 100),
			      (float)download->kps);

   gtk_clist_set_text(GTK_CLIST(download_tab.clist),
		      gtk_clist_find_row_from_data(GTK_CLIST(download_tab.clist),
						   download),
		      3, temp_str);
   g_free(temp_str);
   download->temp_size = 0;

   return 1;
}

int network_update_upload(gpointer data) {
   char *temp_str;
   NAP_DOWNLOAD *upload;
   
   if (!data) return 0;   
   upload = (NAP_DOWNLOAD *)data;
   
   upload->calls++;
   if (upload->calls >= 3) { /* x seconds */
      if (upload->temp_size == 0)
	upload->timeout_calls++;
      else
	upload->timeout_calls = 0;

      upload->kps =
	(((float)upload->temp_size / (float)1024)) / 
	((float)(CLIST_UPDATE_SPEED / 1000) * 3);
      upload->temp_size = 0;
      upload->calls = 0;

     
      if (upload->timeout_calls == ((5 * 60) / 3)) {
	 upload_close(upload, TRANSFER_TIMEOUT);
	 return 0;
      }
   }
   
   temp_str = g_strdup_printf("%.02fM [%i%%] @ %.01fk/s",
			      ((float)upload->size / 1024) / 1024,
			      (int)((float)upload->size / (float)upload->total * 100),
			      (float)upload->kps);
   
   gtk_clist_set_text(GTK_CLIST(upload_tab.clist),
		      gtk_clist_find_row_from_data(GTK_CLIST(upload_tab.clist),
						   upload),
		      3, temp_str);
   g_free(temp_str);
//   upload->temp_size = 0;
   
   return 1;
}

void upload_close(NAP_DOWNLOAD *upload, TRANSFER_INFO info) {
   char *temp_str = NULL;
	 
   gdk_input_remove(upload->sock_input);
   gtk_timeout_remove(upload->update_tag);

   napster_send(conn_info.sock, NAPSTER_UPLOAD_COMPLETE, NULL);
   
   if (info & TRANSFER_CLOSED)
     temp_str = g_strdup_printf("File upload cancelled for %s by %s (%s lost)\n",
				upload->trunc_file, upload->user, simplify_size(upload->total - upload->size));
   else if (info & TRANSFER_FINISHED)
     temp_str = g_strdup_printf("File upload complete for %s to %s\n",
				upload->trunc_file, upload->user);
   else if (info & TRANSFER_TIMEOUT)
     temp_str = g_strdup_printf("File upload timed out for %s to %s\n",
				upload->trunc_file, upload->user);
   
   if (temp_str) {
      gnapster_text_insert(CONSOLE, temp_str);
      g_free(temp_str);
   }
   
   close(upload->sock);
   fclose(upload->fdesc);
   gtk_clist_remove(GTK_CLIST(upload_tab.clist),
		    gtk_clist_find_row_from_data(GTK_CLIST(upload_tab.clist), upload));
}

void network_handle_upload(gpointer data, int source, GdkInputCondition condition) {
   char buf[2048 + 1];
   int n, sent;
   NAP_DOWNLOAD *upload;
   
   upload = (NAP_DOWNLOAD *)data;
   
   if (upload->size < upload->total) {
      lseek(fileno(upload->fdesc), upload->size, SEEK_SET);
      n = read(fileno(upload->fdesc), buf, 1024);
      if (n <= 0) {
	 printf("read() failed\n");
	 upload_close(upload, TRANSFER_CLOSED);
	 return;
      }
      sent = send(upload->sock, buf, n, 0);
   
      if (sent <= 0) {
	 perror("send");
	 upload_close(upload, TRANSFER_CLOSED);
      } else {
	 upload->size += sent;
	 upload->temp_size += sent;
      }
   } else
     upload_close(upload, TRANSFER_FINISHED);
}

void network_conn_user_header_cb(gpointer data, int source, GdkInputCondition condition) {
   NAP_DOWNLOAD *download;
   char c;
   int n;
   
   download = (NAP_DOWNLOAD *)data;

   n = recv(source, &c, 1, 0);
   if (download->firewall) {
      /* Ok lets transfer this code over to the true agent now */
      gdk_input_remove(download->sock_input);
      if (c != '1')
	return;
      n = recv(source, &c, 1, 0);
      if (n <= 0 || !isdigit((unsigned char)c))
	return;

      network_add_upload(download);
      return;
   }
   
   if (n <= 0) {
      download->header[download->header_val] = '\0';
      gtk_label_set_text(GTK_LABEL(download_tab.queue_label), "");
      gnapster_text_insert(CONSOLE, "Connection closed by remote user %s\n",
			   download->user);
      close(source);
      queue_check(download->file, 1);
      gdk_input_remove(download->sock_input);
      download_free(download);
      return;
   }

   if (c == '\n') return;
   
   if (!isdigit((unsigned char)c)) {
      char *temp_str;
      
      download->header[download->header_val] = '\0';
      gdk_input_remove(download->sock_input);
      
      if (!strcmp(download->header, "1") || download->header_val <= 1) {
	 gtk_label_set_text(GTK_LABEL(download_tab.queue_label), "");
	 gnapster_text_insert(CONSOLE, "Could not access file from %s (try again?)\n",
			      download->user);
	 close(source);
	 queue_check(download->file, 1);
	 download_free(download);
	 return;	 
      }
      
      temp_str = download->header;
      temp_str++;
      
      download->total = my_atoi(temp_str);
      
      download->fdesc = open_file(download, download->trunc_file);
      if (!download->fdesc) {
	 gnome_error_dialog(_("Uh oh.  The path you've chosen does not exist or you do not have write permissions\nPlease open up napster's preferences and pick a directory you know you have access to"));
	 close(source);
	 download_free(download);
	 return;
      }
      download->size = 1;
      if (fwrite(&c, 1, sizeof(char), download->fdesc) <= 0) {
	 gnapster_text_insert(CONSOLE, "Error writing to disk!\n");
	 download_close(download, TRANSFER_CANCEL);
	 return;
      }
      network_add_download(download);
   } else {
      download->header[download->header_val++] = c;
   }
}

void network_conn_user_cb(gpointer data, int source, GdkInputCondition condition) {
   NAP_DOWNLOAD *download;
   int err, len;
   
   download = (NAP_DOWNLOAD *)data;

   gdk_input_remove(download->sock_input);

   len = sizeof(err);
   getsockopt(source, SOL_SOCKET, SO_ERROR, &err, &len);
   if (err) {
      gnapster_text_insert(CONSOLE,"* Error connecting to user...\n");
      close(source);
      queue_check(download->file, 1);
      download_free(download);
      return;
   }

   fcntl(source, F_SETFL, 0);   
   
   if (!download->firewall)
     napster_send_get(source, user_info.user, download->file,
		      user_info.connection);
   else {
      int uploads;
      struct stat st;

//      send(source, "1", 1, 0);
      
      uploads = g_list_length(GTK_CLIST(upload_tab.clist)->row_list);

      if (user_info.conf[MAX_UPLOADS_SWITCH] &&
	  uploads >= user_info.conf[MAX_UPLOADS]) {
	 gnapster_text_insert(CONSOLE, "Rejecting upload request for %s to %s, max uploads reached\n",
			      download->trunc_file, download->user);
	 close(download->sock);
	 download_free(download);
	 return;
      } else if (user_info.conf[REJECT_UL] &&
		 g_list_length(GTK_CLIST(download_tab.clist)->row_list)) {
	 gnapster_text_insert(CONSOLE, "Rejecting upload request for %s to %s, active downloads found\n",
			      download->trunc_file, download->user);
	 close(download->sock);
	 download_free(download);
	 return;
      }

      download->fdesc = fopen(conv_unix(download->file), "r");
      if (!download->fdesc) {
	 char *temp_str;
	 
	 temp_str = g_strdup_printf("FILE NOT FOUND");
	 send(download->sock, temp_str, strlen(temp_str), 0);
	 g_free(temp_str);
	 close(download->sock);
	 download_free(download);
	 return;
      }

      fstat(fileno(download->fdesc), &st);
      download->size = 0;
      download->total = st.st_size;

      conv_win(download->file);
      napster_send_send(source, user_info.user, download->file,
			download->total);
   }

   download->sock_input = gdk_input_add(source, GDK_INPUT_READ,
					network_conn_user_header_cb, download);
}

void network_conn_cb(gpointer data, int source, GdkInputCondition condition) {
   int n, port_int;
   char buf[1024];
   char *ip, *port;
   int err;
   int len;
 
   len = sizeof(err);
   getsockopt(source, SOL_SOCKET, SO_ERROR, &err, &len);
   if (err) {
      gdk_input_remove(conn_info.sock_input);
      close(source);
      gnapster_update_status(_("Unknown fatal error connecting to server (try again?)"), 0);
      conn_info.connected = 0;
      conn_info.connecting = 0;
      gtk_widget_set_sensitive(search_tab.media, 0);
      return;
   }

   fcntl(source, F_SETFL, 0);
   
   n = recv(source, buf, 1024, 0);
   if (n <= 0) {
      perror("recv");
      disconnect_cb();
      return;
   }
   ip = strdup(buf);
   port = strchr(ip, ':');
   port[0] = '\0';
   port++;
   
   port_int = atoi(port);
   
   g_free(conn_info.ip);
   conn_info.ip = strdup(ip);
   conn_info.port = port_int;
   
   printf("ip = %s, port = %i\n", conn_info.ip, conn_info.port);
   
   close(source);
   gdk_input_remove(conn_info.sock_input);

   if (!strcmp(conn_info.ip, "127.0.0.1")) {
      gnapster_update_status(_("Napster query server reported all servers were busy"), 0);
      conn_info.connected = 0;
      conn_info.connecting = 0;
      gtk_widget_set_sensitive(search_tab.media, 0);
      return;
   }
   
   gnapster_update_status(_("Best host found, connecting..."), -1);
   napster_connect(conn_info.ip, conn_info.port);
}

void network_read_cb(gpointer data, int source, GdkInputCondition condition) {
   int n;
   unsigned char buf[355];
   unsigned char buf_holder[355];
   int old_n, i;
   PACKET *p;

   if (read_count == 0) {
      total_packet_len = 4;
      packet_len = 0;
      read_count++;
   }

   if (read_count == 1) {
      n = 0;
      bzero(buf, sizeof(buf));
      while(n < 4) {
	 bzero(buf_holder, sizeof(buf_holder));
	 old_n = n;
	 n += recv(source, buf_holder, (4 - n), 0);
	 if (n <= old_n) {
	    n = 0;
	    perror("recv");
	    disconnect_cb();
	    return;
	 }
	 for(i=0; i<(n-old_n); i++) {
	    buf[old_n + i] = buf_holder[i];
	 }
      }
      
      p = g_malloc(sizeof(PACKET));
      memcpy(p, buf, 4);
      p->len = BSWAP16(p->len);
      p->cmd = BSWAP16(p->cmd);
      DPRINTF(("l=%i,c=%i\n", p->len, p->cmd));

      bzero(packet_data, sizeof(packet_data));
      
      packet_len = 0;
      total_packet_len = (int)p->len;
      packet_cmd = (int)p->cmd;
      g_free(p);
      
      read_count++;
      if (packet_len == total_packet_len) read_count++;
   } else if (read_count == 2) {
      int lenbuf;
      bzero(buf_holder, sizeof(buf_holder));
      lenbuf = recv(source, buf_holder, (total_packet_len - packet_len), 0);
      if (lenbuf < 0) {
	 perror("recv");
	 disconnect_cb();
	 return;
      }
      packet_len += lenbuf;
      strcat(packet_data, buf_holder);
      
      if (packet_len >= total_packet_len) read_count++;
   }
   
   if (read_count == 3) {
      DPRINTF(("d=%s\n", packet_data));

      if (!conn_info.connected) {
	 conn_info.connecting = 0;
	 conn_info.connected = 1;
      }

      napster_handle_data(packet_cmd, packet_data);
      
      read_count = 0;
   }
}

void network_conn_real_cb(gpointer data, int source, GdkInputCondition condition) {
   char temp_str[80];
   int err, len;
   
   gdk_input_remove(conn_info.sock_input);

   len = sizeof(err);
   getsockopt(source, SOL_SOCKET, SO_ERROR, &err, &len);
   if (err) {
      gnapster_text_insert(CONSOLE, "Failure connecting to napster server.\n");
      gnapster_update_status(_("Failed to connect"), 0);
      conn_info.connecting = conn_info.connected = 0;
      gtk_widget_set_sensitive(search_tab.media, 0);
      return;
   }

   fcntl(source, F_SETFL, 0);

   if (user_info.new_user)
     napster_send(source, NAPSTER_NEW_USER, user_info.user);
   napster_send(source, NAPSTER_LOGIN, "%s %s %i \"%s\" %i",
		user_info.user, user_info.pass, conn_info.nap_port, NAP_VERSION,
		user_info.connection);
   
   g_snprintf(temp_str, sizeof(temp_str), _("Connected (%s:%i)...awaiting login reply..."),
	      conn_info.ip, conn_info.port);
   gnapster_update_status(temp_str, -1);
   conn_info.sock_input = gdk_input_add(source, GDK_INPUT_READ,
					network_read_cb, NULL);
}
