/* ui.c - user interface creation only */

#include <gnome.h>

#include <netinet/in.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "menus.h"
#include "callbacks.h"

#include "config.h"

extern GNAPSTER_MAIN gnapster_main;
extern SEARCH_TAB search_tab;
extern DOWNLOAD_TAB download_tab;
extern UPLOAD_TAB upload_tab;
extern CONSOLE_TAB console_tab;
extern CDDB_TAB cddb_tab;
extern MOTD_TAB motd_tab;
extern PROP_DLG prop_dlg;
extern BUILD_DLG build_dlg;
extern BROWSE_DLG browse_dlg;
extern ADD_DLG add_dlg;
extern EDIT_DLG edit_dlg;

extern CONN_INFO conn_info;
extern USER_INFO user_info;

GtkWidget *j_line_new(GtkWidget *w, int i1, int i2) {
   GtkWidget *temp;
   
   temp = gtk_hbox_new(FALSE, 5);
   gtk_box_pack_start(GTK_BOX(w), temp, i1, i2, 0);
   
   return temp;
}

GtkWidget *create_gnapster_tab(char *label, char *stock) {
   GtkWidget *vbox;
   GtkWidget *stock_widget;
   GtkWidget *label_widget;
   
   vbox = gtk_hbox_new(FALSE, 0);
   gtk_container_set_border_width(GTK_CONTAINER(vbox), 0);
   
   stock_widget = gnome_stock_pixmap_widget_at_size(gnapster_main.window, stock,
						    15, 15);
   label_widget = gtk_label_new(label);
   
   j_box_pack(vbox, stock_widget, 0, 0, 0);
   j_box_pack(vbox, label_widget, 0, 0, 0);
   
   gtk_widget_show(stock_widget);
   gtk_widget_show(label_widget);
   
   return vbox;
}

void create_gnapster_main() {
   gnapster_main.window = gnome_app_new("gnapster", "Gnapster");
   gtk_widget_set_usize(GTK_WIDGET(gnapster_main.window), 550, 300);
   gtk_window_set_wmclass(GTK_WINDOW(gnapster_main.window),
			  "gnapster", "Gnapster");
   
   gnome_app_create_menus(GNOME_APP(gnapster_main.window), gnapster_menu);
//   gnome_app_create_toolbar(GNOME_APP(gnapster_main.window), gnapster_toolbar);
   
   gtk_window_set_title(GTK_WINDOW(gnapster_main.window), "Gnapster");
   
   gtk_signal_connect(GTK_OBJECT(gnapster_main.window), "delete-event",
		      GTK_SIGNAL_FUNC(gnapster_exit_cb), NULL);
   
   gnapster_main.notebook = gtk_notebook_new();
   gtk_signal_connect(GTK_OBJECT(gnapster_main.notebook), "switch-page",
		      GTK_SIGNAL_FUNC(switch_page_cb), NULL);
   gnome_app_set_contents(GNOME_APP(gnapster_main.window), 
			  gnapster_main.notebook);
   
   gnapster_main.appbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_NEVER);
   gnapster_main.progress = gnome_appbar_get_progress(GNOME_APPBAR(gnapster_main.appbar));
   gtk_progress_set_activity_mode(GTK_PROGRESS(gnapster_main.progress), TRUE);
   gtk_progress_set_value(GTK_PROGRESS(gnapster_main.progress), 0);
   gnome_app_set_statusbar(GNOME_APP(gnapster_main.window),
			   gnapster_main.appbar);
   
   gtk_container_border_width(GTK_CONTAINER(GNOME_APP(gnapster_main.window)->contents), 5);
}

void create_search_tab() {
   char *titles[] = { _("Filename"), _("Connection"), _("User"), _("Size"), _("KBps"), NULL };
   char *vals[] = {
      "At least", "Equal to", "At best",
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN", "Cable", "DSL", "T1", "T3+",
      "Audio", "Video", "Image", "Any",
      NULL
   };
   GList *pop_strings = NULL;
   int i = 0;
   
   search_tab.vbox = gtk_vbox_new(FALSE, 5);

   search_tab.hbox = j_line_new(search_tab.vbox, 0, 0);
   
   search_tab.label = gtk_label_new(_("Query: "));
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.label,
		      FALSE, FALSE, 0);
   
   search_tab.entry = gnome_entry_new("gnapster_search");
   search_tab.gtk_entry = gnome_entry_gtk_entry(GNOME_ENTRY(search_tab.entry));
   gtk_signal_connect(GTK_OBJECT(search_tab.gtk_entry), "activate",
		      GTK_SIGNAL_FUNC(search_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.entry,
		      TRUE, TRUE, 0);

   search_tab.media = gtk_combo_new();
   gtk_widget_set_usize(search_tab.media, 75, -1);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search_tab.media)->entry),
			  FALSE);
   i = 14;
   while (vals[i]) {
      pop_strings = g_list_append(pop_strings, vals[i]);
      i++;
   }
   i = 0;
   gtk_combo_set_popdown_strings(GTK_COMBO(search_tab.media),
				 pop_strings);
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(search_tab.media)->entry), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
   gtk_widget_set_sensitive(search_tab.media, 0);
   j_box_pack(search_tab.hbox, search_tab.media, 0, 0, 0);   

   g_list_free(pop_strings);
   pop_strings = NULL;

   search_tab.search_btn = gtk_button_new_with_label(_("Search"));
   gtk_signal_connect(GTK_OBJECT(search_tab.search_btn), "clicked",
		      GTK_SIGNAL_FUNC(search_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.search_btn,
		      FALSE, FALSE, 0);
   
   search_tab.list_btn = gtk_button_new_with_label(_("Browse user"));
   gtk_signal_connect(GTK_OBJECT(search_tab.list_btn), "clicked",
		      GTK_SIGNAL_FUNC(browse_user_cb), NULL);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.list_btn,
		      FALSE, FALSE, 0);
   
   search_tab.hbox = j_line_new(search_tab.vbox, 0, 0);
   
   search_tab.line_speed_switch = gtk_check_button_new_with_label("Line Speed:");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(search_tab.line_speed_switch),
				user_info.conf[LINE_SPEED_SWITCH]);
   gtk_signal_connect(GTK_OBJECT(search_tab.line_speed_switch), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[LINE_SPEED_SWITCH]));
   j_box_pack(search_tab.hbox, search_tab.line_speed_switch, 0, 0, 0);
   
   search_tab.line_speed_cmp = gtk_combo_new();
   gtk_widget_set_usize(search_tab.line_speed_cmp, 75, -1);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search_tab.line_speed_cmp)->entry),
			  FALSE);
   while (vals[i] && i < 3) {
      pop_strings = g_list_append(pop_strings, vals[i]);
      i++;
   }
   gtk_combo_set_popdown_strings(GTK_COMBO(search_tab.line_speed_cmp),
				 pop_strings);
   gtk_combo_set_value_in_list(GTK_COMBO(search_tab.line_speed_cmp),
			       user_info.conf[LINE_SPEED_CMP], FALSE);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.line_speed_cmp)->entry),
		      listint_to_str(user_info.conf[LINE_SPEED_CMP], 0));
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(search_tab.line_speed_cmp)->entry), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
   j_box_pack(search_tab.hbox, search_tab.line_speed_cmp, 0, 0, 0);
   
   g_list_free(pop_strings);
   pop_strings = NULL;
   
   search_tab.line_speed = gtk_combo_new();
   gtk_widget_set_usize(search_tab.line_speed, 100, -1);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search_tab.line_speed)->entry),
			  FALSE);
   while (vals[i] && i < 14) {
      pop_strings = g_list_append(pop_strings, vals[i]);
      i++;
   }
   gtk_combo_set_popdown_strings(GTK_COMBO(search_tab.line_speed),
				 pop_strings);
   gtk_combo_set_value_in_list(GTK_COMBO(search_tab.line_speed),
			       user_info.conf[LINE_SPEED], FALSE);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.line_speed)->entry),
		      listint_to_str(user_info.conf[LINE_SPEED], 1));
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(search_tab.line_speed)->entry), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
   j_box_pack(search_tab.hbox, search_tab.line_speed, 0, 0, 0);   

   search_tab.label = gtk_label_new("");
   j_box_pack(search_tab.hbox, search_tab.label, 1, 1, 0);
         
   g_list_free(pop_strings);
   pop_strings = NULL;

   search_tab.bitrate_switch = gtk_check_button_new_with_label("Bitrate:");
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(search_tab.bitrate_switch),
				user_info.conf[BITRATE_SWITCH]);
   gtk_signal_connect(GTK_OBJECT(search_tab.bitrate_switch), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[BITRATE_SWITCH]));
   j_box_pack(search_tab.hbox, search_tab.bitrate_switch, 0, 0, 0);

   search_tab.bitrate_cmp = gtk_combo_new();
   gtk_widget_set_usize(search_tab.bitrate_cmp, 75, -1);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search_tab.bitrate_cmp)->entry),
			  FALSE);
   i = 0;
   while (vals[i] && i < 3) {
      pop_strings = g_list_append(pop_strings, vals[i]);
      i++;
   }
   gtk_combo_set_popdown_strings(GTK_COMBO(search_tab.bitrate_cmp),
				 pop_strings);
   gtk_combo_set_value_in_list(GTK_COMBO(search_tab.bitrate_cmp),
			       user_info.conf[BITRATE_CMP], FALSE);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.bitrate_cmp)->entry),
		      g_list_nth_data(pop_strings, user_info.conf[BITRATE_CMP]));
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(search_tab.bitrate_cmp)->entry), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
   j_box_pack(search_tab.hbox, search_tab.bitrate_cmp, 0, 0, 0);

   g_list_free(pop_strings);
   pop_strings = NULL;
   
   search_tab.bitrate = gtk_combo_new();
   gtk_widget_set_usize(search_tab.bitrate, 50, -1);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(search_tab.bitrate)->entry),
			  FALSE);
   
   pop_strings = g_list_append(pop_strings, "96");
   pop_strings = g_list_append(pop_strings, "112");
   pop_strings = g_list_append(pop_strings, "128");
   pop_strings = g_list_append(pop_strings, "160");
   pop_strings = g_list_append(pop_strings, "192");

   gtk_combo_set_popdown_strings(GTK_COMBO(search_tab.bitrate),
				 pop_strings);
   gtk_combo_set_value_in_list(GTK_COMBO(search_tab.bitrate),
			       user_info.conf[BITRATE], FALSE);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(search_tab.bitrate)->entry),
		      g_list_nth_data(pop_strings, user_info.conf[BITRATE]));
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(search_tab.bitrate)->entry), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
   j_box_pack(search_tab.hbox, search_tab.bitrate, 0, 0, 0);
   
   search_tab.hbox = j_line_new(search_tab.vbox, 1, 1);
   
   search_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(search_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(search_tab.hbox), search_tab.scroller,
		      TRUE, TRUE, 0);
   
   search_tab.clist = gtk_clist_new_with_titles(5, titles);
   gtk_clist_set_compare_func(GTK_CLIST(search_tab.clist),
			      (GtkCListCompareFunc)compare_connection);
   gtk_clist_set_sort_column(GTK_CLIST(search_tab.clist), 1);
   gtk_clist_set_selection_mode(GTK_CLIST(search_tab.clist),
				GTK_SELECTION_EXTENDED);
   gtk_clist_set_column_width(GTK_CLIST(search_tab.clist), 0, 250);
   gtk_clist_set_column_width(GTK_CLIST(search_tab.clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(search_tab.clist), 3, 50);
   gtk_container_add(GTK_CONTAINER(search_tab.scroller), search_tab.clist);
   gtk_signal_connect(GTK_OBJECT(search_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), search_popup_menu);
   gtk_signal_connect(GTK_OBJECT(search_tab.clist), "click-column",
		      GTK_SIGNAL_FUNC(search_sort), NULL);

   gtk_widget_set_sensitive(search_tab.line_speed_cmp, user_info.conf[LINE_SPEED_SWITCH]);
   gtk_widget_set_sensitive(search_tab.line_speed, user_info.conf[LINE_SPEED_SWITCH]);
   gtk_widget_set_sensitive(search_tab.bitrate_cmp, user_info.conf[BITRATE_SWITCH]);
   gtk_widget_set_sensitive(search_tab.bitrate, user_info.conf[BITRATE_SWITCH]);

   gtk_container_set_border_width(GTK_CONTAINER(search_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook), 
			    search_tab.vbox, gtk_label_new(_("Search")));
}

void create_download_tab() {
   char *titles[] = { _("Filename"), _("Connection"), _("User"), _("Info"), NULL };
   char *queue_titles[] = { _("Filename"), _("User"), NULL };
   
   download_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   download_tab.hbox = j_line_new(download_tab.vbox, 1, 1);
   
   download_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(download_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(download_tab.hbox), download_tab.scroller,
		      TRUE, TRUE, 0);
   
   download_tab.clist = gtk_clist_new_with_titles(4, titles);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.clist), 0, 235);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.clist), 3, 50);
   gtk_container_add(GTK_CONTAINER(download_tab.scroller), download_tab.clist);
   gtk_signal_connect(GTK_OBJECT(download_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), download_popup_menu);
   
   download_tab.hbox = j_line_new(download_tab.vbox, 0, 0);
   
   download_tab.start_queue = gtk_button_new_with_label(_("Start Queue"));
   gtk_signal_connect(GTK_OBJECT(download_tab.start_queue), "clicked",
		      GTK_SIGNAL_FUNC(start_queue_cb), NULL);
   j_box_pack(download_tab.hbox, download_tab.start_queue, 0, 0, 0);
   
   download_tab.queue_label = gtk_label_new("");
   j_box_pack(download_tab.hbox, download_tab.queue_label, 1, 1, 0);
   
   download_tab.stop_queue = gtk_button_new_with_label(_("Stop Queue"));
   gtk_signal_connect(GTK_OBJECT(download_tab.stop_queue), "clicked",
		      GTK_SIGNAL_FUNC(stop_queue_cb), NULL);
   gtk_widget_set_sensitive(download_tab.stop_queue, 0);
   j_box_pack(download_tab.hbox, download_tab.stop_queue, 0, 0, 0);
   
   download_tab.hbox = j_line_new(download_tab.vbox, 1, 1);
   
   download_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(download_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(download_tab.hbox), download_tab.scroller,
		      TRUE, TRUE, 0);   
   
   download_tab.queue_clist = gtk_clist_new_with_titles(2, queue_titles);
   gtk_clist_set_column_width(GTK_CLIST(download_tab.queue_clist), 0, 325);
   gtk_clist_set_reorderable(GTK_CLIST(download_tab.queue_clist), 1);
   gtk_container_add(GTK_CONTAINER(download_tab.scroller), download_tab.queue_clist);
   gtk_signal_connect(GTK_OBJECT(download_tab.queue_clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), queue_popup_menu);

   gtk_container_set_border_width(GTK_CONTAINER(download_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook),
			    download_tab.vbox, gtk_label_new("Download"));
}

void create_upload_tab() {
   char *titles[] = { _("Filename"), _("Connection"), _("User"), _("Info"), NULL };
   
   upload_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   upload_tab.hbox = j_line_new(upload_tab.vbox, 0, 0);
   
   upload_tab.max_uploads_switch = gtk_check_button_new_with_label(_("Max uploads:"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(upload_tab.max_uploads_switch), 
				user_info.conf[MAX_UPLOADS_SWITCH]);
   gtk_signal_connect(GTK_OBJECT(upload_tab.max_uploads_switch), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[MAX_UPLOADS_SWITCH]));
   j_box_pack(upload_tab.hbox, upload_tab.max_uploads_switch, 0, 0, 0);
   
   upload_tab.max_uploads = gtk_spin_button_new((GtkAdjustment *)gtk_adjustment_new(5.0, 0.0, 50.0, 1.0, 5.0, 0.0),
						0, 0);
   gtk_spin_button_set_value(GTK_SPIN_BUTTON(upload_tab.max_uploads),
			     user_info.conf[MAX_UPLOADS]);
   gtk_signal_connect(GTK_OBJECT(upload_tab.max_uploads), "changed",
		      GTK_SIGNAL_FUNC(combo_changed), NULL);
   j_box_pack(upload_tab.hbox, upload_tab.max_uploads, 0, 0, 0);

   upload_tab.label = gtk_label_new("");
   j_box_pack(upload_tab.hbox, upload_tab.label, 0, 0, 0);

   upload_tab.hbox = j_line_new(upload_tab.vbox, 1, 1);
   
   upload_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(upload_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(upload_tab.hbox), upload_tab.scroller,
		      TRUE, TRUE, 0);
   
   upload_tab.clist = gtk_clist_new_with_titles(4, titles);
   gtk_clist_set_column_width(GTK_CLIST(upload_tab.clist), 0, 235);
   gtk_clist_set_column_width(GTK_CLIST(upload_tab.clist), 2, 75);
   gtk_clist_set_column_width(GTK_CLIST(upload_tab.clist), 3, 50);
   gtk_container_add(GTK_CONTAINER(upload_tab.scroller), upload_tab.clist);
   gtk_signal_connect(GTK_OBJECT(upload_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), upload_popup_menu);
   
   upload_tab.hbox = j_line_new(upload_tab.vbox, 0, 0);
   
   gtk_widget_set_sensitive(upload_tab.max_uploads, user_info.conf[MAX_UPLOADS_SWITCH]);
   
   gtk_container_set_border_width(GTK_CONTAINER(upload_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook),
			    upload_tab.vbox, gtk_label_new(_("Upload")));
}

void create_console_tab() {
   console_tab.main_vbox = gtk_vbox_new(FALSE, 5);
   
   console_tab.notebook = gtk_notebook_new();
   gtk_signal_connect(GTK_OBJECT(console_tab.notebook), "switch-page",
		      GTK_SIGNAL_FUNC(switch_page_cb), NULL);
   j_box_pack(console_tab.main_vbox, console_tab.notebook, 1, 1, 1);

   console_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   console_tab.hbox = j_line_new(console_tab.vbox, 1, 1);
   
   console_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(console_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(console_tab.hbox), console_tab.scroller,
		      TRUE, TRUE, 0);
   
   console_tab.text = gtk_text_new(NULL, NULL);
   gtk_text_set_point(GTK_TEXT(console_tab.text), 0);
   gtk_text_set_editable(GTK_TEXT(console_tab.text), FALSE);
   gtk_container_add(GTK_CONTAINER(console_tab.scroller), console_tab.text);

   console_tab.entry = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(console_tab.entry), "activate",
		      GTK_SIGNAL_FUNC(console_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(console_tab.main_vbox), console_tab.entry,
		      0, 0, 0);
   
   console_tab.channel_list = NULL;

   gtk_container_set_border_width(GTK_CONTAINER(console_tab.vbox), 5);

   gtk_notebook_append_page(GTK_NOTEBOOK(console_tab.notebook),
			    console_tab.vbox, gtk_label_new("Messages"));
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook), 
			    console_tab.main_vbox, gtk_label_new("Console"));
}

void append_console_tab(char *channel_name) {
   char *titles[] = { _("Username"), NULL };
   CHANNEL_TAB channel;
   CHANNEL *chan;
   
   channel.vbox = gtk_vbox_new(FALSE, 5);
   
   channel.hbox = j_line_new(channel.vbox, 1, 1);
   
   channel.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(channel.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(channel.hbox, channel.scroller, 1, 1, 0);
   
   channel.text = gtk_text_new(NULL, NULL);
   gtk_text_set_point(GTK_TEXT(channel.text), 0);
   gtk_text_set_editable(GTK_TEXT(channel.text), FALSE);
   
   gtk_container_add(GTK_CONTAINER(channel.scroller), channel.text);
   channel.scroller2 = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(channel.scroller2),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(channel.hbox, channel.scroller2, 0, 0, 0);
   
   channel.clist = gtk_clist_new_with_titles(1, titles);
   gtk_clist_set_sort_column(GTK_CLIST(channel.clist), 0);
   gtk_widget_set_usize(GTK_WIDGET(channel.clist), 100, -1);
   gtk_container_add(GTK_CONTAINER(channel.scroller2), 
		     channel.clist);
   gtk_signal_connect(GTK_OBJECT(channel.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(menu_popup_cb), chat_popup_menu);
      
   gtk_container_set_border_width(GTK_CONTAINER(channel.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(console_tab.notebook),
			    channel.vbox, gtk_label_new(channel_name));
   
   gtk_widget_show_all(channel.vbox);

   chan = g_malloc0(sizeof(CHANNEL));
   chan->channel_name = strdup(channel_name);
   chan->text = channel.text;
   chan->clist = channel.clist;
   chan->vbox = channel.vbox;
   
   console_tab.channel_list = g_list_append(console_tab.channel_list,
					    chan);
}

void create_cddb_tab() {
   char *titles[] = { _("Matches"), NULL };

   cddb_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   cddb_tab.hbox = j_line_new(cddb_tab.vbox, 0, 0);
   
   cddb_tab.label = gtk_label_new(_("CDDB Search: "));
   gtk_box_pack_start(GTK_BOX(cddb_tab.hbox), cddb_tab.label,
		      FALSE, FALSE, 0);
   
   cddb_tab.entry = gtk_entry_new();
   gtk_signal_connect(GTK_OBJECT(cddb_tab.entry), "activate",
		      GTK_SIGNAL_FUNC(cddb_activate_cb), NULL);
   gtk_box_pack_start(GTK_BOX(cddb_tab.hbox), cddb_tab.entry,
		      TRUE, TRUE, 0);
   
   cddb_tab.hbox = j_line_new(cddb_tab.vbox, 0, 0);
   
   cddb_tab.artist = gtk_check_button_new_with_label(_("Artist"));
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cddb_tab.artist), 1);
   gtk_box_pack_start(GTK_BOX(cddb_tab.hbox), cddb_tab.artist,
		      FALSE, FALSE, 0);
   
   cddb_tab.album = gtk_check_button_new_with_label(_("Album"));
   gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(cddb_tab.album), 1);
   gtk_box_pack_start(GTK_BOX(cddb_tab.hbox), cddb_tab.album,
		      FALSE, FALSE, 0);
   
   cddb_tab.song = gtk_check_button_new_with_label(_("Song"));
   gtk_box_pack_start(GTK_BOX(cddb_tab.hbox), cddb_tab.song,
		      FALSE, FALSE, 0);
   
   cddb_tab.hbox = j_line_new(cddb_tab.vbox, 1, 1);
   
   cddb_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(cddb_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   gtk_box_pack_start(GTK_BOX(cddb_tab.hbox), cddb_tab.scroller,
		      TRUE, TRUE, 0);
   
   cddb_tab.clist = gtk_clist_new_with_titles(1, titles);
   gtk_signal_connect(GTK_OBJECT(cddb_tab.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(cddb_click_cb), NULL);
   gtk_container_add(GTK_CONTAINER(cddb_tab.scroller), cddb_tab.clist);

   gtk_container_set_border_width(GTK_CONTAINER(cddb_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook),
			    cddb_tab.vbox, gtk_label_new(_("CDDB")));
}

void create_motd_tab() {
   motd_tab.vbox = gtk_vbox_new(FALSE, 5);
   
   motd_tab.hbox = j_line_new(motd_tab.vbox, 1, 1);
   
   motd_tab.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(motd_tab.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(motd_tab.hbox, motd_tab.scroller, 1, 1, 0);
   
   motd_tab.text = gtk_text_new(NULL, NULL);
   gtk_text_set_point(GTK_TEXT(motd_tab.text), 0);
   gtk_text_set_editable(GTK_TEXT(motd_tab.text), 0);
   gtk_container_add(GTK_CONTAINER(motd_tab.scroller), motd_tab.text);
   
   gtk_container_set_border_width(GTK_CONTAINER(motd_tab.vbox), 5);
   
   gtk_notebook_append_page(GTK_NOTEBOOK(gnapster_main.notebook), motd_tab.vbox,
			    gtk_label_new(_("Message Of The Day")));
}

void create_prop_dlg(void *changed_cb, void *button_cb, void *destroy_cb,
		     void *clicked_cb, int *new_user_val) {
   char *user, *pass, *download_dir, *upload_dir;
   int conn;
   GList *pop_strings = NULL;
   char *vals[] = {
      "Unknown", "14.4", "28.8", "33.6", "56k", "64k ISDN", "128k ISDN",
      "Cable", "DSL", "T1", "T3+", NULL
   };
   int i = 0;
   
   user = gnome_config_get_string("/gnapster/User/user");
   pass = gnome_config_get_string("/gnapster/User/pass");
   download_dir = gnome_config_get_string("/gnapster/User/dl_dir");
   upload_dir = gnome_config_get_string("/gnapster/User/ul_dir");
   conn = gnome_config_get_int("/gnapster/User/connection");
   
   prop_dlg.calling = 0;
   
   prop_dlg.window = gnome_property_box_new();
   gtk_window_set_title(GTK_WINDOW(prop_dlg.window),
			_("Gnapster Properties"));
   
   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Username: "));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.user_entry = gtk_entry_new();
   if (user) gtk_entry_set_text(GTK_ENTRY(prop_dlg.user_entry), user);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.user_entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.user_entry,
		      FALSE, FALSE, 0);
   
   prop_dlg.new_user = gtk_check_button_new_with_label(_("New Account"));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.new_user), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), new_user_val);
/*   gtk_signal_connect(GTK_OBJECT(prop_dlg.new_user), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);*/
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.new_user,
		      FALSE, FALSE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Password: "));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.pass_entry = gtk_entry_new();
   gtk_entry_set_visibility(GTK_ENTRY(prop_dlg.pass_entry), FALSE);
   if (pass) gtk_entry_set_text(GTK_ENTRY(prop_dlg.pass_entry), pass);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.pass_entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.pass_entry,
		      FALSE, FALSE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Data port: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.nap_port = gtk_entry_new_with_max_length(5);
   if (conn_info.nap_port) {
      char *temp = g_strdup_printf("%i", conn_info.nap_port);
      gtk_entry_set_text(GTK_ENTRY(prop_dlg.nap_port), temp);
      g_free(temp);
   } else
     gtk_entry_set_text(GTK_ENTRY(prop_dlg.nap_port), "6699");
   gtk_widget_set_usize(prop_dlg.nap_port, 50, -1);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.nap_port), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.nap_port, 0, 0, 0);
   
   prop_dlg.firewall = gtk_check_button_new_with_label(_("Firewalled without the ability to accept connections"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.firewall),
				user_info.conf[FIREWALL]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.firewall), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[FIREWALL]));
   j_box_pack(prop_dlg.hbox, prop_dlg.firewall, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);

   prop_dlg.label = gtk_label_new(_("Connection: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);
   
   prop_dlg.connection = gtk_combo_new();
   gtk_widget_set_usize(prop_dlg.connection, 100, -1);
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(prop_dlg.connection)->entry),
			  FALSE);
   while (vals[i]) {
      pop_strings = g_list_append(pop_strings, vals[i]);
      i++;
   }
   gtk_combo_set_popdown_strings(GTK_COMBO(prop_dlg.connection),
				 pop_strings);
   gtk_combo_set_value_in_list(GTK_COMBO(prop_dlg.connection), 1, 0);
   gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(prop_dlg.connection)->entry),
		      napster_conn_as_str(conn));
   gtk_signal_connect(GTK_OBJECT(GTK_COMBO(prop_dlg.connection)->entry), "changed",
		      GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.connection, 0, 0, 0);
   
   g_list_free(pop_strings);
   pop_strings = NULL;
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
  
   prop_dlg.label = gtk_label_new(_("Download Directory: "));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.download_dir = gnome_file_entry_new(NULL, _("Browse Download Directory"));
   if (download_dir) 
     gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.download_dir))), download_dir);
   gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.download_dir))),
		      "changed", GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.download_dir,
		      TRUE, TRUE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.label = gtk_label_new(_("Upload Directory: "));
   j_box_pack(prop_dlg.hbox, prop_dlg.label, 0, 0, 0);

   prop_dlg.upload_dir = gnome_file_entry_new(NULL, _("Browse Upload Directory"));
   if (upload_dir) 
     gtk_entry_set_text(GTK_ENTRY(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir))), upload_dir);
   gtk_signal_connect(GTK_OBJECT(gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY(prop_dlg.upload_dir))),
		      "changed", GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.upload_dir,
		      TRUE, TRUE, 0);

   prop_dlg.build_list = gtk_button_new_with_label(_("Build mp3 list"));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.build_list), "clicked",
		      GTK_SIGNAL_FUNC(build_list_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.build_list, 0, 0, 0);

   prop_dlg.update_list = gtk_button_new_with_label(_("Update list"));
   gtk_signal_connect(GTK_OBJECT(prop_dlg.update_list), "clicked",
		      GTK_SIGNAL_FUNC(update_list_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.update_list, 0, 0, 0);
   
   gnome_property_box_append_page(GNOME_PROPERTY_BOX(prop_dlg.window),
				  prop_dlg.vbox, 
				  gtk_label_new(_("User Information")));

   prop_dlg.vbox = gtk_vbox_new(FALSE, 5);
   gtk_container_border_width(GTK_CONTAINER(prop_dlg.vbox), 5);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.name_fix = gtk_check_button_new_with_label(_("Convert spaces to underscores?"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.name_fix),
				user_info.conf[NAME_FIX]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.name_fix), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[NAME_FIX]));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.name_fix,
		      FALSE, FALSE, 0);

   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.theme_colors = gtk_check_button_new_with_label(_("Use themes state colors for text widget\n(may cause problems with some themes)"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.theme_colors), 
				user_info.conf[THEME_COLORS]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.theme_colors), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[THEME_COLORS]));
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.theme_colors,
		      FALSE, FALSE, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.auto_rm = gtk_check_button_new_with_label(_("Automatically remove cancelled mp3s with 1.5 megabytes or more left (doesnt work)"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.auto_rm), 
				user_info.conf[AUTO_RM]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.auto_rm), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[AUTO_RM]));
//   j_box_pack(prop_dlg.hbox, prop_dlg.auto_rm, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
/*   prop_dlg.opennap_server = gtk_check_button_new_with_label(_("Use development opennap query server:"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.opennap_server), 
				user_info.conf[OPENNAP_SERVER]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.opennap_server), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[OPENNAP_SERVER]));
   j_box_pack(prop_dlg.hbox, prop_dlg.opennap_server, 0, 0, 0);
   
   prop_dlg.server_entry = gnome_entry_new("server_history");
   gnome_entry_load_history(GNOME_ENTRY(prop_dlg.server_entry));
   if (user_info.server) 
     gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(prop_dlg.server_entry))),
			user_info.server);
   gtk_signal_connect(GTK_OBJECT(gnome_entry_gtk_entry(GNOME_ENTRY(prop_dlg.server_entry))),
		      "changed", GTK_SIGNAL_FUNC(changed_cb), NULL);
   j_box_pack(prop_dlg.hbox, prop_dlg.server_entry, 1, 1, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);*/
   
   prop_dlg.auto_start_queue = gtk_check_button_new_with_label(_("Automatically activate queue when nothing is downloading"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.auto_start_queue), 
				user_info.conf[AUTO_START_QUEUE]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.auto_start_queue), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[AUTO_START_QUEUE]));
   j_box_pack(prop_dlg.hbox, prop_dlg.auto_start_queue, 0, 0, 0);
   
   prop_dlg.hbox = j_line_new(prop_dlg.vbox, 0, 0);
   
   prop_dlg.reject_ul = gtk_check_button_new_with_label(_("Reject uploads while downloads are active"));
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(prop_dlg.reject_ul),
				user_info.conf[REJECT_UL]);
   gtk_signal_connect(GTK_OBJECT(prop_dlg.reject_ul), "clicked",
		      GTK_SIGNAL_FUNC(clicked_cb), &(user_info.conf[REJECT_UL]));
   j_box_pack(prop_dlg.hbox, prop_dlg.reject_ul, 0, 0, 0);
   
   gnome_property_box_append_page(GNOME_PROPERTY_BOX(prop_dlg.window),
				  prop_dlg.vbox,
				  gtk_label_new(_("Options")));

/*   prop_dlg.label = gtk_label_new("Connection: ");
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.label,
		      FALSE, FALSE, 0);
   
   prop_dlg.conn_combo = gtk_combo_new();
   gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(prop_dlg.conn_combo)->entry),
			  FALSE);
   while(conn_table[i]) {
      conn_combo_attr = g_list_append(conn_combo_attr, conn_table[i]);
      i++;
   }
   gtk_combo_set_popdown_strings(GTK_COMBO(prop_dlg.conn_combo),
				 conn_combo_attr);
   gtk_signal_connect(GTK_OBJECT(GTK_ENTRY(GTK_COMBO(prop_dlg.conn_combo)->entry)),
		      "changed", GTK_SIGNAL_FUNC(changed_cb), NULL);
   gtk_box_pack_start(GTK_BOX(prop_dlg.hbox), prop_dlg.conn_combo,
		      FALSE, FALSE, 0);*/
   
   gtk_signal_connect(GTK_OBJECT(prop_dlg.window), "apply",
		      GTK_SIGNAL_FUNC(button_cb), "apply");
   gtk_signal_connect(GTK_OBJECT(prop_dlg.window), "destroy",
		      GTK_SIGNAL_FUNC(destroy_cb), "destroy");
}

void create_build_dlg() {
   build_dlg.window = gnome_dialog_new(_("Building MP3 list..."),
				       GNOME_STOCK_BUTTON_CANCEL,
				       NULL);
   
   gtk_widget_set_usize(build_dlg.window, 250, -1);
   
   build_dlg.vbox = gtk_vbox_new(FALSE, 5);
  
   build_dlg.hbox = j_line_new(build_dlg.vbox, 0, 0);
   
   build_dlg.label = gtk_label_new("Building list...");
   j_box_pack(build_dlg.hbox, build_dlg.label, 0, 0, 0);
   
   build_dlg.hbox = j_line_new(build_dlg.vbox, 0, 0);
   
   build_dlg.label = gtk_label_new("");
   j_box_pack(build_dlg.hbox, build_dlg.label, 0, 0, 0);
   
   gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(build_dlg.window)->vbox),
		     build_dlg.vbox);
   
   gnome_dialog_button_connect(GNOME_DIALOG(build_dlg.window), 0,
			       GTK_SIGNAL_FUNC(build_dlg_cancel_cb), NULL);
   
   gtk_widget_show_all(build_dlg.window);
}

void create_browse_dlg() {
   char *titles[] = { "Server title", "Address", NULL };
   
   browse_dlg.window = gnome_dialog_new(_("Browse servers"),
					"Connect",
					GNOME_STOCK_BUTTON_CANCEL,
					NULL);

   gtk_widget_set_usize(browse_dlg.window, 450, 300);

   browse_dlg.vbox = gtk_vbox_new(FALSE, 5);

   browse_dlg.hbox = j_line_new(browse_dlg.vbox, 0, 0);
   
   browse_dlg.add = gtk_button_new_with_label("Add server");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.add), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_add), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.add, 0, 0, 0);
   
   browse_dlg.edit = gtk_button_new_with_label("Edit server");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.edit), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_edit), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.edit, 0, 0, 0);
   
   browse_dlg.delete = gtk_button_new_with_label("Delete server");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.delete), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_delete), NULL);
   j_box_pack(browse_dlg.hbox, browse_dlg.delete, 0, 0, 0);
   
   browse_dlg.refresh = gtk_button_new_with_label("Refresh selected");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.refresh), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_refresh), "selected");
//   j_box_pack(browse_dlg.hbox, browse_dlg.refresh, 0, 0, 0);

   browse_dlg.refresh = gtk_button_new_with_label("Refresh all");
   gtk_signal_connect(GTK_OBJECT(browse_dlg.refresh), "clicked",
		      GTK_SIGNAL_FUNC(browse_dlg_refresh), "all");
//   j_box_pack(browse_dlg.hbox, browse_dlg.refresh, 0, 0, 0);
   
   browse_dlg.hbox = j_line_new(browse_dlg.vbox, 1, 1);

   browse_dlg.scroller = gtk_scrolled_window_new(NULL, NULL);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(browse_dlg.scroller),
				  GTK_POLICY_AUTOMATIC, GTK_POLICY_ALWAYS);
   j_box_pack(browse_dlg.hbox, browse_dlg.scroller, 1, 1, 0);
   
   browse_dlg.clist = gtk_clist_new_with_titles(2, titles);
   gtk_clist_set_column_width(GTK_CLIST(browse_dlg.clist), 0, 225);
//   gtk_clist_set_column_width(GTK_CLIST(browse_dlg.clist), 1, 100);
//   gtk_clist_set_column_width(GTK_CLIST(browse_dlg.clist), 2, 50);
//   gtk_clist_set_column_width(GTK_CLIST(browse_dlg.clist), 3, 75);
   gtk_signal_connect(GTK_OBJECT(browse_dlg.clist), "button_press_event",
		      GTK_SIGNAL_FUNC(browse_dlg_button_cb), NULL);
   populate_browse_dlg_clist();
   gtk_container_add(GTK_CONTAINER(browse_dlg.scroller), browse_dlg.clist);
   
   gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(browse_dlg.window)->vbox),
		     browse_dlg.vbox);
   
   gnome_dialog_button_connect(GNOME_DIALOG(browse_dlg.window), 0,
			       GTK_SIGNAL_FUNC(browse_dlg_connect_cb), NULL);
   
   gnome_dialog_button_connect(GNOME_DIALOG(browse_dlg.window), 1,
			       GTK_SIGNAL_FUNC(browse_dlg_cancel_cb), NULL);
   
   gtk_widget_show_all(browse_dlg.window);
}

void create_add_dlg() {
   add_dlg.window = gnome_dialog_new(_("Add server"),
				     GNOME_STOCK_BUTTON_OK,
				     GNOME_STOCK_BUTTON_CANCEL,
				     NULL);

   add_dlg.vbox = gtk_vbox_new(FALSE, 5);
   
   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);

   add_dlg.label = gtk_label_new("IP: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.server = gtk_entry_new();
   j_box_pack(add_dlg.hbox, add_dlg.server, 1, 1, 0);   

   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);

   add_dlg.label = gtk_label_new("Port: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.port = gtk_entry_new();
   gtk_widget_set_usize(add_dlg.port, 40, -1);
   j_box_pack(add_dlg.hbox, add_dlg.port, 1, 1, 0);   
   add_dlg.hbox = j_line_new(add_dlg.vbox, 0, 0);

   add_dlg.label = gtk_label_new("Description: ");
   j_box_pack(add_dlg.hbox, add_dlg.label, 0, 0, 0);
   
   add_dlg.desc = gtk_entry_new();
   j_box_pack(add_dlg.hbox, add_dlg.desc, 1, 1, 0);   
   
   gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(add_dlg.window)->vbox),
		     add_dlg.vbox);
   
   gnome_dialog_button_connect(GNOME_DIALOG(add_dlg.window), 0,
			       GTK_SIGNAL_FUNC(add_dlg_cb), "ok");
   gnome_dialog_button_connect(GNOME_DIALOG(add_dlg.window), 1,
			       GTK_SIGNAL_FUNC(add_dlg_cb), "cancel");
   
   gtk_widget_show_all(add_dlg.window);
}

void create_edit_dlg(NAP_SERVER *nsrv, char *desc) {
   char *port;
   
   if (!nsrv || !desc) return;
   
   edit_dlg.window = gnome_dialog_new(_("Edit server"),
				     GNOME_STOCK_BUTTON_OK,
				     GNOME_STOCK_BUTTON_CANCEL,
				     NULL);

   edit_dlg.vbox = gtk_vbox_new(FALSE, 5);
   
   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);

   edit_dlg.label = gtk_label_new("IP: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.server = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.server), nsrv->ip);
   j_box_pack(edit_dlg.hbox, edit_dlg.server, 1, 1, 0);   

   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);

   edit_dlg.label = gtk_label_new("Port: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.port = gtk_entry_new();
   port = g_strdup_printf("%i", nsrv->port);
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.port), port);
   g_free(port);
   gtk_widget_set_usize(edit_dlg.port, 40, -1);
   j_box_pack(edit_dlg.hbox, edit_dlg.port, 1, 1, 0);   
   edit_dlg.hbox = j_line_new(edit_dlg.vbox, 0, 0);

   edit_dlg.label = gtk_label_new("Description: ");
   j_box_pack(edit_dlg.hbox, edit_dlg.label, 0, 0, 0);
   
   edit_dlg.desc = gtk_entry_new();
   gtk_entry_set_text(GTK_ENTRY(edit_dlg.desc), desc);
   j_box_pack(edit_dlg.hbox, edit_dlg.desc, 1, 1, 0);   
   
   gtk_container_add(GTK_CONTAINER(GNOME_DIALOG(edit_dlg.window)->vbox),
		     edit_dlg.vbox);
   
   gnome_dialog_button_connect(GNOME_DIALOG(edit_dlg.window), 0,
			       GTK_SIGNAL_FUNC(edit_dlg_cb), nsrv);
   gnome_dialog_button_connect(GNOME_DIALOG(edit_dlg.window), 1,
			       GTK_SIGNAL_FUNC(edit_dlg_cb), NULL);
   
   gtk_widget_show_all(edit_dlg.window);
}
