#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <sys/wait.h>
#include <netinet/in.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>

#include <dirent.h>

#include <time.h>

#include <gnome.h>

#include "napster.h"
#include "gnapster.h"
#include "ui.h"
#include "callbacks.h"

#include "upload.h"
#include "md5.h"

#include "network.h"

#ifndef MAP_FAILED
#define MAP_FAILED (void *) -1
#endif

extern CONN_INFO conn_info;
extern USER_INFO user_info;
extern UPLOAD_TAB upload_tab;
extern BUILD_DLG build_dlg;

extern int building;

extern GList *total_list;
extern int submit_index;

/* I owe panasync a lot for this code in here :) */

static unsigned char _buffer[32];
static int _bptr = 0;

int _get_input(int fd, unsigned char *bp, int size) {
   if (read(fd, bp, size) != size)
     return -1;
   return 0;
}

static int readsync(int fd) {
   _bptr = 0;
   _buffer[0] = _buffer[1];
   _buffer[1] = _buffer[2];
   _buffer[2] = _buffer[3];
   
   return _get_input(fd, &_buffer[3], 1);
}

static int _fillbfr(int fd, unsigned int size) {
   _bptr = 0;
   return _get_input(fd, _buffer, size);
}

static unsigned int _getbits(int n) {
   unsigned int pos, ret_value;
   
   pos = _bptr >> 3;
   ret_value = 
     _buffer[pos] << 24 |
     _buffer[pos + 1] << 16 |
     _buffer[pos + 2] << 8 |
     _buffer[pos + 3];
   ret_value <<= _bptr & 7;
   ret_value >>= 32 - n;
   _bptr += n;
   
   return ret_value;
}

static void parse_header(HEADER *header) {
   header->idex = _getbits(1);
   header->id = _getbits(1);
   header->layer = _getbits(2);
   header->protection_bit = _getbits(1);
   header->bitrate_index = _getbits(4);
   header->freq = _getbits(2);
   header->padding_bit = _getbits(1);
   header->private_bit = _getbits(1);
   header->mode = _getbits(2);
   header->mode_extension = _getbits(2);
   if (!header->mode)
     header->mode_extension = 0;
   header->copyright = _getbits(1);
   header->original = _getbits(1);
   header->emphasis = _getbits(2);
   
   header->stereo = (header->mode == 3) ? 1 : 2;
   header->true_layer = 4 - header->layer;
}

int get_header(int fd, HEADER *header) {
   int retval;
   
   if ((retval = _fillbfr(fd, 4)))
     return retval;
   
   while(_getbits(11) != 0x7ff) {
      if ((retval = readsync(fd)) != 0)
	return retval;
   }
   
   parse_header(header);
   
   return 0;
}

long get_bitrate(int fd, time_t *mp3_time, int *freq_rate,
		 unsigned long *filesize, int *stereo, long *id3, int *mime_type, 
		 time_t *mtime) {
   short bitrates[2][3][15] = {
      { 
	 { 0, 32, 48, 56, 64, 80, 96, 112, 128, 144, 160, 176, 192, 224, 256 },
	 { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160 },
	 { 0, 8, 16, 24, 32, 40, 48, 56, 64, 80, 96, 112, 128, 144, 160 }
      },
      {
	 { 0, 32, 64, 96, 128, 160, 192, 224, 256, 288, 320, 352, 384, 416, 448 },
	 { 0, 32, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320, 384 },
	 { 0, 32, 40, 48, 56, 64, 80, 96, 112, 128, 160, 192, 224, 256, 320 }
      }
   };
   
   int frequencies[2][2][3] = {
      {
	 { 11025, 12000, 8000 },
	 { 0, 0, 0 },
      },
      {
	 { 22050, 24000, 16000 },
	 { 44100, 48000, 32000 },
      }
   };
   
   HEADER header = { 0 };
   unsigned long btr = 0;
   
   struct stat st;
   unsigned long framesize = 0, totalframes = 0;
   
   if (freq_rate)
     *freq_rate = 0;
   
   fstat(fd, &st);
   
   if (!(*filesize = st.st_size))
     return 0;
   
   *mtime = st.st_mtime;
   
   get_header(fd, &header);
   if (header.id > 1 || header.layer > 2 || header.bitrate_index > 14)
     return 0;
   
   btr = bitrates[header.id][3 - header.layer][header.bitrate_index];
   
   if (frequencies[header.idex][header.id][header.freq] > 0)
     framesize = (header.id ? 144000 : 72000) * btr / frequencies[header.idex][header.id][header.freq];
   totalframes = (st.st_size / (framesize + 1)) - 1;
   if (frequencies[header.idex][header.id][header.freq] > 0) 
     *mp3_time = (time_t) (totalframes * (header.id == 0 ? 576 : 1152) / frequencies[header.idex][header.id][header.freq]);
   
   if (freq_rate)
     *freq_rate = frequencies[header.idex][header.id][header.freq];
   if (id3) {
      char buf[130];
      int rc;
      lseek(fd, 0, SEEK_SET);
      *id3 = 0;
      rc = read(fd, buf, 128);
      if (!strncmp(buf, "ID3", 3)) {
	 struct id3v2 {
	    char tag[3];
	    unsigned char ver[2];
	    unsigned char flag;
	    unsigned char size[4];
	 } id3v2;
	 unsigned char bytes[4];
	 
	 memcpy(&id3v2, buf, sizeof(id3v2));
	 bytes[3] = id3v2.size[3] | ((id3v2.size[2] & 1) << 7);
	 bytes[2] = ((id3v2.size[2] >> 1) & 63) | ((id3v2.size[1] & 3) << 6);
	 bytes[1] = ((id3v2.size[1] >> 2) & 31) | ((id3v2.size[0] & 7) << 5);
	 bytes[0] = ((id3v2.size[0] >> 3) & 15);
	 
	 *id3 = (bytes[3] | (bytes[2] << 8) | (bytes[1] << 16) | (bytes[0] << 24)) + sizeof(struct id3v2);
      }
      lseek(fd, st.st_size - 128, SEEK_SET);
      rc = read(fd, buf, 128);
      if ((rc == 128) && !strncmp(buf, "TAG", 3))
	*id3 = *id3 ? (*id3 * -1) : 1;
   }
   *stereo = header.mode;
   
   return btr;
}

char *get_checksum(int fd, unsigned long mapsize) {
   unsigned char digest[16];
   md5_state_t state;
   char buffer[2048 + 1];
   struct stat st;
   unsigned long size = 3000000;
   int di = 0;
   int rc;
   char *m;
   
   *buffer = 0;
   md5_init(&state);
   if ((fstat(fd, &st)) == -1) 
     return strdup("");
   
   if (!mapsize) {
      if (st.st_size < size)
	size = st.st_size;
   } else if (st.st_size < mapsize)
	size = st.st_size;
   else size = mapsize;
   
   if ((m = mmap((void *)0, size, PROT_READ, MAP_PRIVATE, fd, 0)) != MAP_FAILED) {
      md5_append(&state, (unsigned char *)m, size);
      md5_finish(digest, &state);
      munmap(m, size);
      memset(buffer, 0, 200);
      for(di=0, rc=0; di<16; ++di, rc+=2)
	g_snprintf(&buffer[rc], 2048, "%02x", digest[di]);
      strcat(buffer, "-");
      sprintf(buffer, "%s%li", buffer, st.st_size);
   }
   
   return strdup(buffer);
}

char *conv_win(char *s) {
   char *ptr;
   
   ptr = s;
   if (!ptr || !*ptr) return NULL;
   
   while(*ptr) {
      if (*ptr == '/') *ptr = '\\';
      ptr++;
   }
   
   return s;
}

char *conv_unix(char *s) {
   char *ptr;
   
   ptr = s;
   if (!ptr || !*ptr) return NULL;
   
   while(*ptr) {
      if (*ptr == '\\') *ptr = '/';
      ptr++;
   }
   
   return s;
}

GList *list_match(GList *list, char *file) {
   GList *ptr;

   if (!list) return NULL;
   if (!file) return NULL;
   
   ptr = total_list;
   while(ptr) {
      if (!strcmp(conv_unix(((MP3 *)ptr->data)->filename), file))
	return ptr;
      ptr = ptr->next;
   }
   
   return NULL;
}

int check_ext(char *p) {
   char *ext;
   
   ext = strrchr(p, '.');
   if (!ext) return 0;
   
   if (strcasecmp(ext, ".mp3"))
     return 0;
   
   return 1;
}

void process_file(char *p, struct stat st, int pd, FILE *fd, int flag, GList *list) {
   int f;
   GList *match = NULL;
   MP3 *mp3, *match_mp3 = NULL;
   long id3 = 0;
   char *buffer;
   int build = 1;

   if (!p) return;
   
   if (!check_ext(p)) return;
   if (flag == UPDATE_LIST) {
      match = list_match(list, p);
      if (match) {
	 match_mp3 = (MP3 *)match->data;
	 if (!match_mp3) return;

	 if (st.st_mtime <= match_mp3->mtime)
	   build = 0;
      }
   }
   if (build) {
      write(pd, p, strlen(p));

      f = open(p, O_RDONLY);
      mp3 = g_malloc0(sizeof(MP3));
      mp3->filename = strdup(p);
      mp3->bitrate = get_bitrate(f, &mp3->time, &mp3->freq, &mp3->filesize,
				 &mp3->stereo, &id3, &mp3->type, &mp3->mtime);
      if (mp3->filesize && mp3->bitrate) {
	 unsigned long size = 300000;
	 switch(id3) {
	  case 1:
	    if (mp3->filesize < size)
	      size = mp3->filesize - 128;
	  case 0:
	    lseek(f, 0, SEEK_SET);
	    break;
	  default:
	    lseek(f, (id3 < 0) ? -id3 : id3, SEEK_SET);
	    if (id3 > 0) {
	       if ((mp3->filesize - id3) < size)
		 size = mp3->filesize - id3;
	    } else {
	       if ((mp3->filesize + id3 - 128) < size)
		 size = mp3->filesize + id3 - 128;
	    }
	    break;
	 }
	 mp3->checksum = get_checksum(f, size);
	 close(f);
	 buffer = g_strdup_printf("\"%s\" %s %lu %u %u %lu", conv_win(mp3->filename),
				  mp3->checksum, mp3->filesize, mp3->bitrate,
				  mp3->freq, mp3->time);
	 printf("d=%s\n", buffer);
	 fprintf(fd, "%lu %s\n", mp3->mtime, buffer);
	 g_free(buffer);

	 if (!building) return;

	 g_free(mp3->checksum);
      }
      g_free(mp3->filename);
      g_free(mp3);
   } else {
      if (!match_mp3) return;

      /* Not rebuilding data, simply writing what we have... */
      fprintf(fd, "%lu \"%s\" %s %lu %u %u %lu\n", match_mp3->mtime,
	      conv_win(match_mp3->filename), match_mp3->checksum, 
	      match_mp3->filesize, match_mp3->bitrate, match_mp3->freq, 
	      match_mp3->time);
   }
   if (match) {
      GList *link;
      
      link = g_list_find(total_list, match->data);
      total_list = g_list_remove_link(total_list, link);
      g_free(match_mp3->filename);
      g_free(match_mp3->checksum);
      g_free(match_mp3);
   }
}

void rodir(int pd, FILE *fd, char *n, int flags, GList *list) {
   DIR *dir;
   char p[4096];
   struct dirent *dptr;
   struct stat st;

   dir = opendir(n);
   if (!dir) return;
   for(;;) {
      dptr = readdir(dir);
      if (dptr == NULL) break;
      g_snprintf(p, sizeof(p), "%s/%s", n, dptr->d_name);
      stat(p, &st);
      if (strcmp(dptr->d_name, ".") && strcmp(dptr->d_name, "..")) {
	 if (S_ISDIR(st.st_mode))
	   rodir(pd, fd, p, flags, list);
	 else {
	    if (!building) return;
	    process_file(p, st, pd, fd, flags, list);
	 }
      }
   }
}

void build_pipe_stop(int sig) {
   building = 0;
}

void build_pipe_read(gpointer data, int source, GdkInputCondition condition) {
   char buf[2048];
   char *file;
   int n;
   
   n = read(source, buf, 2048 - 1);
   if (n <= 0) {
      if (building)
	build_dlg_cancel_cb();
   }
   
   buf[n] = '\0';
   file = strrchr(buf, '/');
   if (!file) {
      if (!strcmp(buf, "BUILD_DONE")) {
	 printf("build done\n");
	 build_dlg.cpid = 0;
	 build_dlg_cancel_cb();
      }
      return;
   } else if (!strcmp(file, "/BUILD_DONE")) {
      printf("build done\n");
      build_dlg.cpid = 0;
      build_dlg_cancel_cb();
      return;
   }
   file++;
   gtk_label_set_text(GTK_LABEL(build_dlg.label), file);
}

void setup_build_dlg(FILE *f, char *path, int flags, GList *list) {
   int pd[2], cpid;
   
   building = 1;
   pipe(pd);
   if ((cpid = fork()) == 0) {
      close(pd[0]);
      signal(SIGUSR1, build_pipe_stop);
      rodir(pd[1], f, path, flags, list);
      write(pd[1], "/BUILD_DONE", strlen("/BUILD_DONE"));
      fclose(f);
      _exit(0);
   }
   fclose(f);   
   build_dlg.cpid = cpid;
   create_build_dlg();
   build_dlg.pipe_input = gdk_input_add(pd[0], GDK_INPUT_READ,
					build_pipe_read, NULL);
   gnome_dialog_run_and_close(GNOME_DIALOG(build_dlg.window));
   wait(NULL);
}

void remove_slash(char *s) {
   char *ptr;
   
   if (!s) return;
   ptr = strrchr(s, '/');
   if (strlen(ptr) == 1) 
     ptr[0] = 0;
}

FILE *open_shared(int status) {
   FILE *f;
   char *conf, *conf2;
   struct stat st;
   int x = -1;
   
   conf = g_strdup_printf("%s/.gnapster", getenv("HOME"));
   if (stat(conf, &st) < 0) {
      x = mkdir(conf, 0755);
      if (x < 0) {
	 perror("mkdir");
	 return NULL;
      }
      /* It made the dir and succeeded... */
      g_free(conf);
      conf2 = g_strdup_printf("%s/.gnapster_shared", getenv("HOME"));
      if (stat(conf2, &st) < 0) return NULL;
      conf = g_strdup_printf("%s/.gnapster/shared", getenv("HOME"));
      rename(conf2, conf);
      g_free(conf2);
   }
   g_free(conf);
   
   conf = g_strdup_printf("%s/.gnapster/shared", getenv("HOME"));
   f = fopen(conf, (status) ? "w" : "r");
   if (!f) {
      perror("fopen");
      return NULL;
   }
   g_free(conf);
   
   return f;
}

int parse_shared_entry(char *s, time_t *mtime, char **file,
		       char **md5, unsigned long int *size, 
		       long *bitrate, unsigned long int *freq,
		       time_t *time) {
   char *mtime_str, *size_str, *bitrate_str, *freq_str, *time_str;

   mtime_str = next_arg(s, &s);
   *file = next_arg(s, &s);
   *md5 = next_arg(s, &s);
   size_str = next_arg(s, &s);
   bitrate_str = next_arg(s, &s);
   freq_str = next_arg(s, &s);
   time_str = next_arg(s, &s);
   
   if (!time_str) return -1;
   
   sscanf(mtime_str, "%li", mtime);
   sscanf(size_str, "%lu", size);
   sscanf(bitrate_str, "%li", bitrate);
   sscanf(freq_str, "%lu", freq);
   sscanf(time_str, "%li", time);
   
   return 1;
}

void shared_list_handle(char *path, int flags) {
//   GList *total_files = NULL;
   FILE *f;
   
   if (flags == SUBMIT_LIST)
     if (!conn_info.connected || conn_info.connecting)
       return;
   
   if (!path) return;
   remove_slash(path);
   
   if (flags != UPDATE_LIST)
     user_info.sharing = user_info.share_size = 0;
   
   if (flags != BUILD_LIST) {
      char buf[256];
      unsigned long int size, freq;
      char *file = NULL, *md5 = NULL;
      long int bitrate;
      time_t mtime, time;
      MP3 *mp3;
      
      f = open_shared(0);
      if (!f) return;
      
      /* Don't let the list already exist at this point */
      total_list = mp3_list_free(total_list);
      submit_index = 0;
      while(!feof(f)) {
	 fgets(buf, sizeof(buf) - 1, f);
	 if (feof(f)) break;
	 if (parse_shared_entry(buf, &mtime, &file, &md5, &size, &bitrate, &freq, &time) < 0)
	   return;
	 
	 if (flags == SUBMIT_LIST) {
	    user_info.sharing++;
	    user_info.share_size += size;
	 }
	 
	 mp3 = g_malloc0(sizeof(MP3));
	 mp3->filename = strdup(file);
	 mp3->checksum = strdup(md5);
	 mp3->filesize = size;
	 mp3->bitrate = bitrate;
	 mp3->freq = freq;
	 mp3->time = time;
	 mp3->mtime = mtime;

	 total_list = g_list_append(total_list, mp3);
      }
      fclose(f);
      
      if (flags == SUBMIT_LIST) {
	 char *temp_str;
	 temp_str = g_strdup_printf("Sharing %i (%.02f gigabytes)", user_info.sharing,
				    (((float)user_info.share_size / (float)1024) / (float)1024) / (float)1024);
	 gtk_label_set_text(GTK_LABEL(upload_tab.label), temp_str);
	 g_free(temp_str);
	 
//	 gnapster_update_status("Submitting MP3 list to server...", 1);
	 conn_info.upload_input = gdk_input_add(conn_info.sock, GDK_INPUT_WRITE,
						network_handle_submit, total_list);
      }
   } else if (file_exists(path)) {
      f = open_shared(1);
      if (!f) return;
      
      setup_build_dlg(f, path, flags, NULL);
   }
   if (total_list && flags == UPDATE_LIST) {
      GList *ptr;
      
      f = open_shared(1);
      if (!f) return;
      
      setup_build_dlg(f, path, flags, total_list);
      
      ptr = total_list;
      while(ptr) {
	 if (ptr->data) {
	    MP3 *mp3ptr;
	    
	    mp3ptr = (MP3 *)ptr->data;
	    g_free(mp3ptr->filename);
	    g_free(mp3ptr->checksum);
	    g_free(mp3ptr);
	    mp3ptr = NULL;
	 }
	 ptr = ptr->next;
      }
      g_list_free(total_list);
      total_list = NULL;
   }
}
